@@ -34,12 +34,15 @@
 // Local Includes
 #include "adblocksettingwidget.h"
 
+#include "webpage.h"
+
 // KDE Includes
 #include <KIO/FileCopyJob>
 #include <KStandardDirs>
 
 // Qt Includes
 #include <QUrl>
+#include <QTimer>
 #include <QWebElement>
 #include <QNetworkReply>
 #include <QNetworkRequest>
@@ -66,7 +69,8 @@ AdBlockManager::AdBlockManager(QObject *parent)
     , _isAdblockEnabled(false)
     , _isHideAdsEnabled(false)
 {
-    loadSettings();
+    // NOTE: launch this with 1 sec delay to get sure we can start up fast...
+    QTimer::singleShot(1000, this, SLOT(loadSettings()));
 }
 
 
@@ -113,6 +117,7 @@ void AdBlockManager::loadSettings()
 
     _whiteList.clear();
     _blackList.clear();
+
     _hideList.clear();
 
     KConfigGroup settingsGroup(_adblockConfig, "Settings");
@@ -208,6 +213,9 @@ void AdBlockManager::loadRuleString(const QString &stringRule)
             return;
 
         const QString filter = stringRule.mid(2);
+        if (filter.isEmpty())
+            return;
+        
         AdBlockRule rule(filter);
         _whiteList << rule;
         return;
@@ -216,7 +224,11 @@ void AdBlockManager::loadRuleString(const QString &stringRule)
     // hide (CSS) rules
     if (stringRule.startsWith(QL1S("##")))
     {
-        _hideList << stringRule.mid(2);
+        const QString filter = stringRule.mid(2);
+        if (filter.isEmpty())
+            return;
+        
+        _hideList << filter;
         return;
     }
 
@@ -302,7 +314,7 @@ void AdBlockManager::updateSubscription(int i)
     QString rulesFilePath = KStandardDirs::locateLocal("appdata" , QL1S("adblockrules_") + n);
     KUrl destUrl = KUrl(rulesFilePath);
 
-    KIO::FileCopyJob* job = KIO::file_copy(subUrl , destUrl, -1, KIO::HideProgressInfo);
+    KIO::FileCopyJob* job = KIO::file_copy(subUrl , destUrl, -1, KIO::HideProgressInfo | KIO::Overwrite);
     job->metaData().insert("ssl_no_client_cert", "TRUE");
     job->metaData().insert("ssl_no_ui", "TRUE");
     job->metaData().insert("UseCache", "false");
@@ -380,3 +392,51 @@ bool AdBlockManager::isAdblockEnabledForHost(const QString &host)
 {
     return ! _hostWhiteList.match(host);
 }
+
+
+void AdBlockManager::applyHidingRules(QWebFrame *frame)
+{
+    if (!frame)
+        return;
+
+    if (!_isAdblockEnabled)
+        return;
+
+    connect(frame, SIGNAL(loadFinished(bool)), this, SLOT(applyHidingRules(bool)));
+}
+
+
+void AdBlockManager::applyHidingRules(bool ok)
+{
+    if (!ok)
+        return;
+    
+    QWebFrame *frame = qobject_cast<QWebFrame *>(sender());
+    if (!frame)
+        return;
+
+    WebPage *page = qobject_cast<WebPage *>(frame->page());
+    if (!page)
+        return;
+    
+    QString mainPageHost = page->loadingUrl().host();
+    QStringList hosts = ReKonfig::whiteReferer();
+    if (hosts.contains(mainPageHost))
+        return;
+    
+    QWebElement document = frame->documentElement();
+
+    // HIDE RULES
+    Q_FOREACH(const QString & filter, _hideList)
+    {
+        QWebElementCollection elements = document.findAll(filter);
+
+        Q_FOREACH(QWebElement el, elements)
+        {
+            if (el.isNull())
+                continue;
+            kDebug() << "Hide element: " << el.localName();
+            el.removeFromDocument();
+        }
+    }
+}
