@@ -38,7 +38,7 @@
 #include "searchengine.h"
 
 #include "tabbar.h"
-#include "tabwindow.h"
+#include "rekonqwindow.h"
 
 #include "webwindow.h"
 #include "webtab.h"
@@ -69,15 +69,24 @@
 
 #include <KMessageBox>
 
+#include <config-kactivities.h>
+#ifdef HAVE_KACTIVITIES
+#include <KActivities/Consumer>
+#endif
+
 // Qt Includes
 #include <QDBusInterface>
 #include <QDBusReply>
 #include <QDir>
+#include <QTimer>
 
 
 Application::Application()
     : KUniqueApplication()
 {
+#ifdef HAVE_KACTIVITIES
+    m_activityConsumer = new KActivities::Consumer();
+#endif
 }
 
 
@@ -88,6 +97,10 @@ Application::~Application()
     ReKonfig::setRecoverOnCrash(0);
     saveConfiguration();
 
+#ifdef HAVE_KACTIVITIES    
+    delete m_activityConsumer;
+#endif
+
     kDebug() << "Bye bye (k)baby...";
 }
 
@@ -137,9 +150,7 @@ int Application::newInstance()
         }
         kDebug() << "URL: " << u;
 
-        WebTab *tab = newWebApp();
-        connect(tab->page(), SIGNAL(pageCreated(WebPage*)), this, SLOT(pageCreated(WebPage*)));
-        tab->view()->load(u);
+        loadUrl(u, Rekonq::WebApp);
 
         if (isFirstLoad)
         {
@@ -181,20 +192,30 @@ int Application::newInstance()
         if (isFirstLoad)
         {
             bool restoreOk = false;
-            if (ReKonfig::startupBehaviour() == 2)
-            {
-                restoreOk = SessionManager::self()->restoreSessionFromScratch();
-            }
-            else
+            
+            switch(ReKonfig::startupBehaviour())
             {
+            case 2:
+                if (hasToBeRecoveredFromCrash)
+                    restoreOk = false;
+                else
+                    restoreOk = SessionManager::self()->restoreSessionFromScratch();
+                break;
+                
+            case 3:
+                SessionManager::self()->manageSessions();
+                break;
+                
+            default:
                 restoreOk = SessionManager::self()->restoreJustThePinnedTabs();
+                break;
             }
 
             isFirstLoad = !restoreOk;
         }
 
         // first argument: 99% of the time we have just that...
-        if (isFirstLoad || m_tabWindows.count() == 0)
+        if (isFirstLoad || m_rekonqWindows.count() == 0)
         {
             // No windows in the current desktop? No windows at all?
             // Create a new one and load there sites...
@@ -218,8 +239,8 @@ int Application::newInstance()
                 loadUrl(urlList.at(0), Rekonq::NewWindow);
             }
 
-            if (!tabWindow()->isActiveWindow())
-                KWindowSystem::demandAttention(tabWindow()->winId(), true);
+            if (!rekonqWindow()->isActiveWindow())
+                KWindowSystem::demandAttention(rekonqWindow()->winId(), true);
         }
 
         // following arguments: what's best behavior here?
@@ -263,26 +284,30 @@ int Application::newInstance()
                 case 1: // open new tab page
                     if (incognito)
                     {
-                        loadUrl(KUrl("about:home"), Rekonq::NewPrivateWindow);
+                        loadUrl(KUrl("rekonq:home"), Rekonq::NewPrivateWindow);
                         break;
                     }
                     if (SessionManager::self()->restoreJustThePinnedTabs())
-                        loadUrl(KUrl("about:home") , Rekonq::NewTab);
+                        loadUrl(KUrl("rekonq:home") , Rekonq::NewTab);
                     else
-                        loadUrl(KUrl("about:home"), Rekonq::NewWindow);
+                        loadUrl(KUrl("rekonq:home"), Rekonq::NewWindow);
                     break;
                 case 2: // restore session
                     if (incognito)
                     {
-                        loadUrl(KUrl("about:home"), Rekonq::NewPrivateWindow);
+                        loadUrl(KUrl("rekonq:home"), Rekonq::NewPrivateWindow);
                         break;
                     }
-                    if (SessionManager::self()->restoreSessionFromScratch())
+                    if (hasToBeRecoveredFromCrash || !SessionManager::self()->restoreSessionFromScratch())
                     {
-                        break;
+                        loadUrl(KUrl("rekonq:home") , Rekonq::NewTab);
                     }
+                    break;
+                case 3:
+                    SessionManager::self()->manageSessions();
+                    break;
                 default:
-                    newTabWindow()->newTab();
+                    ASSERT_NOT_REACHED(unknown startup behavior);
                     break;
                 }
             }
@@ -294,7 +319,7 @@ int Application::newInstance()
             switch (ReKonfig::newTabsBehaviour())
             {
             case 0: // new tab page
-                loadUrl(KUrl("about:home") , type);
+                loadUrl(KUrl("rekonq:home") , type);
                 break;
             case 2: // homepage
                 loadUrl(KUrl(ReKonfig::homePage()) , type);
@@ -312,8 +337,8 @@ int Application::newInstance()
     {
         if (hasToBeRecoveredFromCrash && !incognito)
         {
-            if (tabWindow() && tabWindow()->currentWebWindow())
-                QTimer::singleShot(1000, tabWindow()->currentWebWindow(), SLOT(showCrashMessageBar()));
+            if (rekonqWindow() && rekonqWindow()->currentWebWindow())
+                QTimer::singleShot(1000, rekonqWindow()->currentWebWindow(), SLOT(showCrashMessageBar()));
         }
         else
         {
@@ -321,7 +346,7 @@ int Application::newInstance()
         }
 
         if (ReKonfig::checkDefaultSearchEngine() && !hasToBeRecoveredFromCrash && SearchEngine::defaultEngine().isNull())
-            QTimer::singleShot(2000, tabWindow()->currentWebWindow()->tabView(), SLOT(showSearchEngineBar()));
+            QTimer::singleShot(2000, rekonqWindow()->currentWebWindow()->tabView(), SLOT(showSearchEngineBar()));
 
         // updating rekonq configuration
         updateConfiguration();
@@ -354,81 +379,98 @@ void Application::saveConfiguration() const
 }
 
 
-TabWindow *Application::tabWindow()
+// --------------------------------------------------------------------------------------------------------------------
+
+
+RekonqWindow *Application::rekonqWindow(const QString & activityID)
 {
-    TabWindow *active = qobject_cast<TabWindow*>(QApplication::activeWindow());
+    RekonqWindow *active = qobject_cast<RekonqWindow*>(QApplication::activeWindow());
 
     if (!active)
     {
-        if (m_tabWindows.isEmpty())
+        RekonqWindowList wList = m_rekonqWindows;
+#ifdef HAVE_KACTIVITIES
+        wList = tabsForActivity(activityID);
+#endif
+        
+        if (wList.isEmpty())
             return 0;
-
-        Q_FOREACH(const QWeakPointer<TabWindow> &pointer, m_tabWindows)
+        
+        Q_FOREACH(const QWeakPointer<RekonqWindow> &pointer, wList)
         {
             if (KWindowInfo(pointer.data()->effectiveWinId(), NET::WMDesktop, 0).isOnCurrentDesktop())
                 return pointer.data();
         }
-        return m_tabWindows.at(0).data();
+        return wList.at(0).data();
     }
     return active;
 }
 
 
-void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
+RekonqWindowList Application::tabsForActivity(const QString & activityID)
 {
-    if (url.isEmpty())
-        return;
-
-    if (!url.isValid())
-    {
-        KMessageBox::error(0, i18n("Malformed URL:\n%1", url.url(KUrl::RemoveTrailingSlash)));
-        return;
-    }
-
-    Rekonq::OpenType newType = type;
-    // Don't open useless tabs or windows for actions in about: pages
-    if (url.url().contains("about:") && url.url().contains("/"))
-        newType = Rekonq::CurrentTab;
-
-    TabWindow *w = 0;
-    if (newType == Rekonq::NewPrivateWindow)
-    {
-        w = newTabWindow(true, true);
-        newType = Rekonq::CurrentTab;
-    }
-    else if (newType == Rekonq::NewWindow
-             || ((newType == Rekonq::NewTab || newType == Rekonq::NewFocusedTab) && tabWindowList().count() == 0))
-    {
-        w = newTabWindow();
-        newType = Rekonq::CurrentTab;
-    }
-    else
-    {
-        w = tabWindow();
-    }
-
-    w->loadUrl(url, newType);
+#ifdef HAVE_KACTIVITIES
+    QString id = activityID;
+    if ( id.isEmpty() )
+        id = m_activityConsumer->currentActivity();
+    
+    return m_activityRekonqWindowsMap[id];
+#else
+    return m_rekonqWindows;
+#endif
 }
 
 
-TabWindow *Application::newTabWindow(bool withTab, bool PrivateBrowsingMode)
+bool Application::haveWindowsForActivity(const QString & activityID)
 {
-    TabWindow *w = new TabWindow(withTab, PrivateBrowsingMode);
+    return (!tabsForActivity(activityID).isEmpty());
+}
 
+
+// -----------------------------------------------------------------------------------------------------------
+
+
+RekonqWindow *Application::newWindow(bool withTab, bool PrivateBrowsingMode)
+{
+    RekonqWindow *w = new RekonqWindow(withTab, PrivateBrowsingMode);
+    setWindowInfo(w);
+    w->show();
+
+    return w;
+}
+
+
+RekonqWindow *Application::newWindow(WebPage *pg)
+{
+    RekonqWindow *w = new RekonqWindow(pg);
+    setWindowInfo(w);
+    w->show();
+
+    return w;    
+}
+
+
+void Application::setWindowInfo(RekonqWindow *w)
+{
     // set object name
-    int n = m_tabWindows.count() + 1;
+    int n = m_rekonqWindows.count() + 1;
     w->setObjectName(QL1S("win") + QString::number(n));
 
     // This is used to track which window was activated most recently
     w->installEventFilter(this);
 
-    m_tabWindows.prepend(w);
-    w->show();
-
-    return w;
+    m_rekonqWindows.prepend(w);
+    
+#ifdef HAVE_KACTIVITIES
+    QString currentActivity = m_activityConsumer->currentActivity();
+    m_activityRekonqWindowsMap[currentActivity].prepend(w);
+#endif    
 }
 
 
+// -----------------------------------------------------------------------------------------------------------
+
+
 WebTab *Application::newWebApp()
 {
     WebTab *tab = new WebTab;
@@ -442,9 +484,9 @@ WebTab *Application::newWebApp()
 }
 
 
-TabWindowList Application::tabWindowList()
+RekonqWindowList Application::rekonqWindowList()
 {
-    return m_tabWindows;
+    return m_rekonqWindows;
 }
 
 
@@ -454,38 +496,49 @@ WebAppList Application::webAppList()
 }
 
 
+// ------------------------------------------------------------------------------------------------------------------
+
+
 bool Application::eventFilter(QObject* watched, QEvent* event)
 {
     // Track which window was activated most recently to prefer it on window choosing
     // (e.g. when another application opens a link)
     if (event->type() == QEvent::WindowActivate)
     {
-        TabWindow *window = qobject_cast<TabWindow*>(watched);
+        RekonqWindow *window = qobject_cast<RekonqWindow*>(watched);
         if (window)
         {
-            if (!m_tabWindows.isEmpty() 
-                && m_tabWindows.at(0) 
-                && m_tabWindows.at(0).data() != window)
+            if (!m_rekonqWindows.isEmpty() 
+                && m_rekonqWindows.at(0) 
+                && m_rekonqWindows.at(0).data() != window)
             {
-                int index = m_tabWindows.indexOf(QWeakPointer<TabWindow>(window));
+                int index = m_rekonqWindows.indexOf(QWeakPointer<RekonqWindow>(window));
                 Q_ASSERT(index != -1);
-                m_tabWindows.prepend(m_tabWindows.takeAt(index));
+                m_rekonqWindows.prepend(m_rekonqWindows.takeAt(index));
             }
         }
     }
 
     // As we are filtering the events occurred to the tabwindows, check also
     // when we close one of them, remove from tab window list and check if it was last...
-    if (event->type() == QEvent::Close)
+    if ((event->type() == QEvent::Close) && !rApp->sessionSaving())
     {
-        TabWindow *window = qobject_cast<TabWindow*>(watched);
-        if (window)
-            m_tabWindows.removeOne(window);
+        RekonqWindow *window = qobject_cast<RekonqWindow*>(watched);
 
+        if (window)
+        {
+            SessionManager::self()->saveSession();
+            m_rekonqWindows.removeOne(window);
+#ifdef HAVE_KACTIVITIES
+            QString currentActivity = m_activityConsumer->currentActivity();
+            m_activityRekonqWindowsMap[currentActivity].removeOne(window);
+#endif
+        }
+        
         WebTab *webApp = qobject_cast<WebTab*>(watched);
         m_webApps.removeOne(webApp);
 
-        if (m_tabWindows.count() == 0 && m_webApps.count() == 0)
+        if (m_rekonqWindows.count() == 0 && m_webApps.count() == 0)
             quit();
     }
 
@@ -493,11 +546,61 @@ bool Application::eventFilter(QObject* watched, QEvent* event)
 }
 
 
+void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
+{
+    if (url.isEmpty())
+        return;
+
+    if (!url.isValid())
+    {
+        KMessageBox::error(0, i18n("Malformed URL:\n%1", url.url(KUrl::RemoveTrailingSlash)));
+        return;
+    }
+
+    if (type == Rekonq::WebApp)
+    {
+        WebTab *tab = newWebApp();
+        connect(tab->page(), SIGNAL(pageCreated(WebPage*)), this, SLOT(pageCreated(WebPage*)));
+        tab->view()->load(url);
+        return;
+    }
+    
+    Rekonq::OpenType newType = type;
+    // Don't open useless tabs or windows for actions in rekonq: pages
+    if (url.url().contains("rekonq:") && url.url().contains("/"))
+        newType = Rekonq::CurrentTab;
+
+    RekonqWindow *w = 0;
+    if (newType == Rekonq::NewPrivateWindow)
+    {
+        w = newWindow(true, true);
+        newType = Rekonq::CurrentTab;
+    }
+    else if (newType == Rekonq::NewWindow
+             || ((newType == Rekonq::NewTab || newType == Rekonq::NewFocusedTab) && !haveWindowsForActivity()))
+    {
+        w = newWindow();
+        newType = Rekonq::CurrentTab;
+    }
+    else
+    {
+        w = rekonqWindow();
+        if (!w)
+        {
+            w = newWindow();
+            newType = Rekonq::CurrentTab;
+        }
+    }
+
+    w->loadUrl(url, newType);
+}
+
+
 void Application::updateConfiguration()
 {
     // ============== Tabs ==================
     bool b = ReKonfig::closeTabSelectPrevious();
-    Q_FOREACH(const QWeakPointer<TabWindow> &w, m_tabWindows)
+    Q_FOREACH(const QWeakPointer<RekonqWindow> &w, m_rekonqWindows)
     {
         if (b)
             w.data()->tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab);
@@ -519,9 +622,9 @@ void Application::updateConfiguration()
     // compute font size
     // (I have to admit I know nothing about these DPI questions..: copied from kwebkitpart, as someone suggested)
     // font size in pixels =  font size in inches × screen dpi
-    if (tabWindow() && tabWindow()->currentWebWindow())
+    if (rekonqWindow() && rekonqWindow()->currentWebWindow())
     {
-        int logDpiY = tabWindow()->currentWebWindow()->logicalDpiY();
+        int logDpiY = rekonqWindow()->currentWebWindow()->logicalDpiY();
         float toPix = (logDpiY < 96.0)
                       ? 96.0 / 72.0
                       : logDpiY / 72.0 ;
@@ -565,7 +668,7 @@ void Application::updateConfiguration()
         QString path = KStandardDirs::locateLocal("cache", QString("WebkitLocalStorage/rekonq"), true);
         path.remove("rekonq");
         QWebSettings::setOfflineStoragePath(path);
-        QWebSettings::setOfflineStorageDefaultQuota(50000);
+        QWebSettings::setOfflineStorageDefaultQuota(ReKonfig::offlineWebApplicationCacheQuota() * 1024);
     }
 
     // ================= WebGl ===================
@@ -580,13 +683,13 @@ void Application::updateConfiguration()
 
     defaultSettings = 0;
 
-    if (!tabWindow())
+    if (!rekonqWindow())
         return;
 
     // FIXME What about this?
 //     ReKonfig::useFavicon()
-//     ? tabWindow()->changeWindowIcon(tabWindow()->mainView()->currentIndex())
-//     : tabWindow()->setWindowIcon(KIcon("rekonq"))
+//     ? rekonqWindow()->changeWindowIcon(rekonqWindow()->mainView()->currentIndex())
+//     : rekonqWindow()->setWindowIcon(KIcon("rekonq"))
 //     ;
 //
     // hovering unfocused tabs options
@@ -594,23 +697,23 @@ void Application::updateConfiguration()
     {
     case 0: // tab previews
     case 3: // nothing
-        for (int i = 0; i < tabWindow()->tabBar()->count(); i++)
+        for (int i = 0; i < rekonqWindow()->tabBar()->count(); i++)
         {
-            tabWindow()->tabBar()->setTabToolTip(i, QL1S(""));
+            rekonqWindow()->tabBar()->setTabToolTip(i, QL1S(""));
         }
         break;
 
     case 1: // title previews
-        for (int i = 0; i < tabWindow()->tabBar()->count(); i++)
+        for (int i = 0; i < rekonqWindow()->tabBar()->count(); i++)
         {
-            tabWindow()->tabBar()->setTabToolTip(i, tabWindow()->tabText(i).remove('&'));
+            rekonqWindow()->tabBar()->setTabToolTip(i, rekonqWindow()->tabWidget()->tabText(i).remove('&'));
         }
         break;
 
     case 2: // url previews
-        for (int i = 0; i < tabWindow()->tabBar()->count(); i++)
+        for (int i = 0; i < rekonqWindow()->tabBar()->count(); i++)
         {
-            tabWindow()->tabBar()->setTabToolTip(i, tabWindow()->webWindow(i)->url().toMimeDataString());
+            rekonqWindow()->tabBar()->setTabToolTip(i, rekonqWindow()->tabWidget()->webWindow(i)->url().toMimeDataString());
         }
         break;
 
@@ -626,14 +729,14 @@ void Application::queryQuit()
 {
     if (m_webApps.count() > 0)
     {
-        tabWindow()->close();
+        rekonqWindow()->close();
         return;
     }
 
-    if (tabWindowList().count() > 1)
+    if (rekonqWindowList().count() > 1)
     {
         int answer = KMessageBox::questionYesNoCancel(
-                         tabWindow(),
+                         rekonqWindow(),
                          i18n("Do you want to close the window or the whole application?"),
                          i18n("Application/Window closing..."),
                          KGuiItem(i18n("C&lose Current Window"),
@@ -646,7 +749,7 @@ void Application::queryQuit()
         switch (answer)
         {
         case KMessageBox::Yes:
-            tabWindow()->close();
+            rekonqWindow()->close();
             return;
 
         case KMessageBox::No:
@@ -657,14 +760,14 @@ void Application::queryQuit()
         }
     }
 
-    // in case of just one window...
+    SessionManager::self()->setSessionManagementEnabled(false);    
     quit();
 }
 
 
 void Application::clearPrivateData()
 {
-    QPointer<KDialog> dialog = new KDialog(tabWindow());
+    QPointer<KDialog> dialog = new KDialog(rekonqWindow());
     dialog->setCaption(i18nc("@title:window", "Clear Private Data"));
     dialog->setButtons(KDialog::Ok | KDialog::Cancel);
 
@@ -739,12 +842,20 @@ void Application::clearPrivateData()
 }
 
 
-void Application::createWebAppShortcut()
+void Application::createWebAppShortcut(const QString & urlString, const QString & titleString)
 {
-    KUrl u = tabWindow()->currentWebWindow()->url();
+    KUrl u;
+    if (urlString.isEmpty())
+    {
+        u = rekonqWindow()->currentWebWindow()->url();
+    }
+    else
+    {
+        u = KUrl(urlString);
+    }
     QString h = u.host();
 
-    QPointer<KDialog> dialog = new KDialog(tabWindow());
+    QPointer<KDialog> dialog = new KDialog(rekonqWindow());
     dialog->setCaption(i18nc("@title:window", "Create Application Shortcut"));
     dialog->setButtons(KDialog::Ok | KDialog::Cancel);
     dialog->button(KDialog::Ok)->setText(i18n("Create"));
@@ -755,7 +866,17 @@ void Application::createWebAppShortcut()
     QWidget widget;
     wAppWidget.setupUi(&widget);
 
-    QString webAppTitle = tabWindow()->currentWebWindow()->title().remove('&');
+    QString webAppTitle;
+    if (titleString.isEmpty())
+    {
+        webAppTitle = rekonqWindow()->currentWebWindow()->title();
+    }
+    else
+    {
+        webAppTitle = titleString;
+    }
+    webAppTitle = webAppTitle.remove('&');
+    
     wAppWidget.nameLineEdit->setText(webAppTitle);
     wAppWidget.kcfg_createDesktopAppShortcut->setChecked(ReKonfig::createDesktopAppShortcut());
     wAppWidget.kcfg_createMenuAppShortcut->setChecked(ReKonfig::createMenuAppShortcut());
@@ -841,33 +962,21 @@ void Application::bookmarksToolbarToggled(bool b)
 
 void Application::newPrivateBrowsingWindow()
 {
-    // NOTE: what about an "about:incognito" page?
-    loadUrl(KUrl("about:home"), Rekonq::NewPrivateWindow);
+    // NOTE: what about a "rekonq:incognito" page?
+    loadUrl(KUrl("rekonq:home"), Rekonq::NewPrivateWindow);
 }
 
 
 void Application::pageCreated(WebPage *pg)
 {
-    if (m_tabWindows.isEmpty())
+    if (m_rekonqWindows.isEmpty())
     {
-        // NOTE: This is "adjusted" from newTabWindow() code...
-        TabWindow *w = new TabWindow(pg);
-
-        // set object name
-        int n = m_tabWindows.count() + 1;
-        w->setObjectName(QL1S("win") + QString::number(n));
-
-        // This is used to track which window was activated most recently
-        w->installEventFilter(this);
-
-        m_tabWindows.prepend(w);
-        w->show();
-
+        newWindow(pg);
         return;
     }
 
-    TabWindow *tw = tabWindow();
-    tw->newTab(pg);
+    RekonqWindow *tw = rekonqWindow();
+    tw->tabWidget()->newTab(pg);
 
     tw->activateWindow();
     tw->raise();
