@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2009-2012 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2013 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
 *
 *
@@ -34,7 +34,8 @@
 
 // Local Includes
 #include "application.h"
-#include "tabwindow.h"
+#include "rekonqwindow.h"
+#include "tabwidget.h"
 
 #include "previewselectorbar.h"
 #include "thumbupdater.h"
@@ -85,6 +86,7 @@ NewTabPage::NewTabPage(QWebFrame *frame)
     {
         m_html = file.readAll();
         m_html.replace(QL1S("$DEFAULT_PATH"), dataPath);
+        m_html.replace(QL1S("$GENERAL_FONT"), QWebSettings::globalSettings()->fontFamily(QWebSettings::StandardFont));
     }
 }
 
@@ -96,8 +98,8 @@ void NewTabPage::generate(const KUrl &url)
     WebView *view = qobject_cast<WebView *>(pg->parent());
     WebTab *tab = view->parentTab();
 
-    // about:preview links
-    if (KUrl("about:settings").isParentOf(url))
+    // rekonq:preview links
+    if (KUrl("rekonq:settings").isParentOf(url))
     {
         if (url.fileName() == QL1S("network"))
         {
@@ -130,8 +132,8 @@ void NewTabPage::generate(const KUrl &url)
         }
     }
 
-    // about:preview links
-    if (KUrl("about:preview").isParentOf(url))
+    // rekonq:preview links
+    if (KUrl("rekonq:preview").isParentOf(url))
     {
         if (url.fileName() == QL1S("add"))
         {
@@ -146,7 +148,7 @@ void NewTabPage::generate(const KUrl &url)
             ReKonfig::setPreviewNames(names);
             ReKonfig::setPreviewUrls(urls);
 
-            loadPageForUrl(KUrl("about:favorites"));
+            loadPageForUrl(KUrl("rekonq:favorites"));
 
             tab->createPreviewSelectorBar(index);
             return;
@@ -174,39 +176,39 @@ void NewTabPage::generate(const KUrl &url)
         }
     }
 
-    // about:closedTabs links
-    if (KUrl("about:closedTabs").isParentOf(url))
+    // rekonq:closedtabs links
+    if (KUrl("rekonq:closedtabs").isParentOf(url))
     {
         if (url.fileName() == QL1S("restore"))
         {
             const int tabIndex = url.queryItem(QL1S("tab")).toInt();
 
-            rApp->tabWindow()->restoreClosedTab(tabIndex, false);
+            rApp->rekonqWindow()->tabWidget()->restoreClosedTab(tabIndex, false);
             return;
         }
     }
 
-    // about:history links
-    if (KUrl("about:history").isParentOf(url))
+    // rekonq:history links
+    if (KUrl("rekonq:history").isParentOf(url))
     {
         if (url.fileName() == QL1S("clear"))
         {
             HistoryManager::self()->clear();
-            loadPageForUrl(KUrl("about:history"));
+            loadPageForUrl(KUrl("rekonq:history"));
             return;
         }
 
         if (url.fileName() == QL1S("showAllItems"))
         {
             m_showFullHistory = true;
-            loadPageForUrl(KUrl("about:history"));
+            loadPageForUrl(KUrl("rekonq:history"));
             return;
         }
 
         if (url.fileName() == QL1S("search"))
         {
             QString value = url.queryItemValue(QL1S("q"));
-            loadPageForUrl(KUrl("about:history"), value);
+            loadPageForUrl(KUrl("rekonq:history"), value);
             return;
         }
 
@@ -214,25 +216,25 @@ void NewTabPage::generate(const KUrl &url)
         {
             int value = url.queryItemValue(QL1S("location")).toInt();
             HistoryManager::self()->removeHistoryLocationEntry(value);
-            loadPageForUrl(KUrl("about:history"));
+            loadPageForUrl(KUrl("rekonq:history"));
             return;
         }
     }
 
-    // about:downloads links
-    if (KUrl("about:downloads").isParentOf(url))
+    // rekonq:downloads links
+    if (KUrl("rekonq:downloads").isParentOf(url))
     {
         if (url.fileName() == QL1S("clear"))
         {
             DownloadManager::self()->clearDownloadsHistory();
-            loadPageForUrl(KUrl("about:downloads"));
+            loadPageForUrl(KUrl("rekonq:downloads"));
             return;
         }
 
         if (url.fileName() == QL1S("search"))
         {
             QString value = url.queryItemValue(QL1S("q"));
-            loadPageForUrl(KUrl("about:downloads"), value);
+            loadPageForUrl(KUrl("rekonq:downloads"), value);
             return;
         }
 
@@ -248,25 +250,25 @@ void NewTabPage::generate(const KUrl &url)
         {
             int value = url.queryItemValue(QL1S("item")).toInt();
             DownloadManager::self()->removeDownloadItem(value);
-            loadPageForUrl(KUrl("about:downloads"));
+            loadPageForUrl(KUrl("rekonq:downloads"));
             return;
         }
     }
 
-    if (url == KUrl("about:bookmarks/edit"))
+    if (url == KUrl("rekonq:bookmarks/edit"))
     {
         BookmarkManager::self()->slotEditBookmarks();
         return;
     }
 
 
-    if (url == KUrl("about:favorites/save"))
+    if (url == KUrl("rekonq:favorites/save"))
     {
         saveFavorites();
         return;
     }
 
-
+    kDebug() << "URL: " << url;
     loadPageForUrl(url);
 }
 
@@ -289,7 +291,7 @@ void NewTabPage::loadPageForUrl(const KUrl &url, const QString & filter)
 
     QString title;
     QByteArray encodedUrl = url.toEncoded();
-    if (encodedUrl == QByteArray("about:favorites"))
+    if (encodedUrl == QByteArray("rekonq:favorites"))
     {
         favoritesPage();
 //         updateWindowIcon();
@@ -298,25 +300,25 @@ void NewTabPage::loadPageForUrl(const KUrl &url, const QString & filter)
         initJS();
         return;
     }
-    else if (encodedUrl == QByteArray("about:history"))
+    else if (encodedUrl == QByteArray("rekonq:history"))
     {
         historyPage(filter);
 //         updateWindowIcon();
         title = i18n("History");
     }
-    else if (encodedUrl == QByteArray("about:bookmarks"))
+    else if (encodedUrl == QByteArray("rekonq:bookmarks"))
     {
         bookmarksPage();
 //         updateWindowIcon();
         title = i18n("Bookmarks");
     }
-    else if (encodedUrl == QByteArray("about:downloads"))
+    else if (encodedUrl == QByteArray("rekonq:downloads"))
     {
         downloadsPage(filter);
 //         updateWindowIcon();
         title = i18n("Downloads");
     }
-    else if (encodedUrl == QByteArray("about:closedTabs"))
+    else if (encodedUrl == QByteArray("rekonq:closedtabs"))
     {
         closedTabsPage();
 //         updateWindowIcon();
@@ -337,31 +339,31 @@ void NewTabPage::browsingMenu(const KUrl &currentUrl)
 
     // Favorites
     navItems.append(createLinkItem(i18n("Favorites"),
-                                   QL1S("about:favorites"),
+                                   QL1S("rekonq:favorites"),
                                    QL1S("emblem-favorite"),
                                    KIconLoader::Toolbar));
 
     // Bookmarks
     navItems.append(createLinkItem(i18n("Bookmarks"),
-                                   QL1S("about:bookmarks"),
+                                   QL1S("rekonq:bookmarks"),
                                    QL1S("bookmarks"),
                                    KIconLoader::Toolbar));
 
     // History
     navItems.append(createLinkItem(i18n("History"),
-                                   QL1S("about:history"),
+                                   QL1S("rekonq:history"),
                                    QL1S("view-history"),
                                    KIconLoader::Toolbar));
 
     // Downloads
     navItems.append(createLinkItem(i18n("Downloads"),
-                                   QL1S("about:downloads"),
+                                   QL1S("rekonq:downloads"),
                                    QL1S("download"),
                                    KIconLoader::Toolbar));
 
     // Closed Tabs
     navItems.append(createLinkItem(i18n("Closed Tabs"),
-                                   QL1S("about:closedTabs"),
+                                   QL1S("rekonq:closedtabs"),
                                    QL1S("tab-close"),
                                    KIconLoader::Toolbar));
 
@@ -372,7 +374,7 @@ void NewTabPage::browsingMenu(const KUrl &currentUrl)
 
         if (it.findFirst(aTagString).attribute(hrefAttributeString) == currentUrl.toMimeDataString())
             it.addClass(QL1S("current"));
-        else if (currentUrl == QL1S("about:home") && it.findFirst(aTagString).attribute(hrefAttributeString) == QL1S("about:favorites"))
+        else if (currentUrl == QL1S("rekonq:home") && it.findFirst(aTagString).attribute(hrefAttributeString) == QL1S("rekonq:favorites"))
             it.addClass(QL1S("current"));
         m_root.document().findFirst(QL1S("#navigation")).appendInside(it);
     }
@@ -384,7 +386,7 @@ void NewTabPage::favoritesPage()
     m_root.addClass(QL1S("favorites"));
 
     QWebElement add = createLinkItem(i18n("Add Favorite"),
-                                     QL1S("about:preview/add"),
+                                     QL1S("rekonq:preview/add"),
                                      QL1S("list-add"),
                                      KIconLoader::Toolbar);
     add.setAttribute(QL1S("class"), QL1S("right"));
@@ -417,20 +419,20 @@ void NewTabPage::historyPage(const QString & filter)
 {
     m_root.addClass(QL1S("history"));
 
-    QWebElement searchForm = createFormItem(i18n("Search History"), QL1S("about:history/search"));
+    QWebElement searchForm = createFormItem(i18n("Search History"), QL1S("rekonq:history/search"));
     searchForm.setAttribute(QL1S("class"), QL1S("left"));
 
     m_root.document().findFirst(QL1S("#actions")).appendInside(searchForm);
 
     QWebElement clearHistory = createLinkItem(i18n("Clear History"),
-                               QL1S("about:history/clear"),
+                               QL1S("rekonq:history/clear"),
                                QL1S("edit-clear"),
                                KIconLoader::Toolbar);
     clearHistory.setAttribute(QL1S("class"), QL1S("right"));
     m_root.document().findFirst(QL1S("#actions")).appendInside(clearHistory);
 
     HistoryTreeModel *model = HistoryManager::self()->historyTreeModel();
-    UrlFilterProxyModel *proxy = new UrlFilterProxyModel(this);
+    SortFilterProxyModel *proxy = new SortFilterProxyModel(this);
     proxy->setSourceModel(model);
 
     bool filterIsEmpty = filter.isEmpty();
@@ -456,6 +458,8 @@ void NewTabPage::historyPage(const QString & filter)
     int i = 0;
     const int maxTextSize = 103;
     const int truncateSize = 100;
+    const QString removeIconPath = QL1S("file:///") + KIconLoader::global()->iconPath("edit-delete", KIconLoader::DefaultState);
+    QWebElement historyItemElement = markup(QL1S(".historyitem"));
     do
     {
         QModelIndex index = proxy->index(i, 0, QModelIndex());
@@ -471,7 +475,7 @@ void NewTabPage::historyPage(const QString & filter)
                 QModelIndex son = proxy->index(j, 0, index);
                 KUrl u = son.data(HistoryModel::UrlStringRole).toUrl();
 
-                historyFolderElement.appendInside(markup(QL1S(".historyitem")));
+                historyFolderElement.appendInside(historyItemElement.clone());
                 QWebElement item = historyFolderElement.lastChild();
 
                 item.findFirst(QL1S(".greytext")).setPlainText( son.data(HistoryModel::DateTimeRole).toDateTime().toString("hh:mm") );
@@ -492,13 +496,12 @@ void NewTabPage::historyPage(const QString & filter)
                 linkElement.appendInside(shownUrl);
 
                 QWebElement removeElement = item.findFirst(QL1S(".button img"));
-                removeElement.setAttribute(QL1S("src"), QL1S("file:///") +
-                                                        KIconLoader::global()->iconPath("edit-delete", KIconLoader::DefaultState));
+                removeElement.setAttribute(QL1S("src"), removeIconPath);
 
                 QWebElement removeLinkElement = item.findFirst(QL1S(".button"));
 
                 int histLoc = HistoryManager::self()->historyFilterModel()->historyLocation(u.url());
-                removeLinkElement.setAttribute(QL1S("href"), QL1S("about:history/remove?location=") + QString::number(histLoc));
+                removeLinkElement.setAttribute(QL1S("href"), QL1S("rekonq:history/remove?location=") + QString::number(histLoc));
 
                 historyFolderElement.appendInside(QL1S("<br />"));
             }
@@ -508,7 +511,7 @@ void NewTabPage::historyPage(const QString & filter)
         {
             m_root.appendInside(markup(QL1S("a")));
             m_root.lastChild().setAttribute(QL1S("class") , QL1S("greybox"));
-            m_root.lastChild().setAttribute(QL1S("href") , QL1S("about:history/showAllItems"));
+            m_root.lastChild().setAttribute(QL1S("href") , QL1S("rekonq:history/showAllItems"));
             m_root.lastChild().setPlainText(i18n("Show full History"));
             return;
         }
@@ -524,7 +527,7 @@ void NewTabPage::bookmarksPage()
     m_root.addClass(QL1S("bookmarks"));
 
     QWebElement editBookmarks = createLinkItem(i18n("Edit Bookmarks"),
-                                QL1S("about:bookmarks/edit"),
+                                QL1S("rekonq:bookmarks/edit"),
                                 QL1S("bookmarks-organize"),
                                 KIconLoader::Toolbar);
     editBookmarks.setAttribute(QL1S("class"), QL1S("right"));
@@ -542,12 +545,18 @@ void NewTabPage::bookmarksPage()
 
     m_root.appendInside(markup(QL1S(".bookmarkfolder")));
     QWebElement rootFolder = m_root.lastChild();
-    rootFolder.appendInside(markup(QL1S("h4")));
-    rootFolder.lastChild().setPlainText(i18n("Unsorted"));
+    rootFolder.appendInside(markup(QL1S("a")));
+    rootFolder.lastChild().setAttribute(QL1S("href"), QL1S("javascript: toggleChildren(\'Unsorted\')"));
+    QWebElement titleElement = rootFolder.lastChild();
+    titleElement.appendInside(markup(QL1S("h4")));    
+    titleElement.lastChild().setPlainText(i18n("Unsorted"));
+
+    rootFolder.appendInside(markup(QL1S("div")));
+    rootFolder.lastChild().setAttribute(QL1S("id"), QL1S("Unsorted"));
 
     while (!bookmark.isNull())
     {
-        createBookmarkItem(bookmark, rootFolder);
+        createBookmarkItem(bookmark, rootFolder.lastChild());
         bookmark = bookGroup.next(bookmark);
     }
 }
@@ -555,9 +564,9 @@ void NewTabPage::bookmarksPage()
 
 void NewTabPage::closedTabsPage()
 {
-    m_root.addClass(QL1S("closedTabs"));
+    m_root.addClass(QL1S("closedtabs"));
 
-    QList<TabHistory> links = rApp->tabWindow()->recentlyClosedTabs();
+    QList<TabHistory> links = rApp->rekonqWindow()->tabWidget()->recentlyClosedTabs();
 
     if (links.isEmpty())
     {
@@ -591,12 +600,12 @@ void NewTabPage::downloadsPage(const QString & filter)
 {
     m_root.addClass(QL1S("downloads"));
 
-    QWebElement searchForm = createFormItem(i18n("Search Downloads"), QL1S("about:downloads/search"));
+    QWebElement searchForm = createFormItem(i18n("Search Downloads"), QL1S("rekonq:downloads/search"));
     searchForm.setAttribute(QL1S("class"), QL1S("left"));
     m_root.document().findFirst(QL1S("#actions")).appendInside(searchForm);
 
     QWebElement clearDownloads = createLinkItem(i18n("Clear Downloads"),
-                                 QL1S("about:downloads/clear"),
+                                 QL1S("rekonq:downloads/clear"),
                                  QL1S("edit-clear"),
                                  KIconLoader::Toolbar);
     clearDownloads.setAttribute(QL1S("class"), QL1S("right"));
@@ -675,7 +684,7 @@ void NewTabPage::downloadsPage(const QString & filter)
             {
                 div.appendInside(markup(QL1S("a")));
                 div.lastChild().setAttribute(QL1S("class"), QL1S("greylink"));
-                div.lastChild().setAttribute(QL1S("href"), QL1S("about:downloads/opendir?q=") + QL1S("file://") + dir);
+                div.lastChild().setAttribute(QL1S("href"), QL1S("rekonq:downloads/opendir?q=") + QL1S("file://") + dir);
                 div.lastChild().setPlainText(i18n("Open directory"));
 
                 div.appendInside(QL1S(" - "));
@@ -694,7 +703,7 @@ void NewTabPage::downloadsPage(const QString & filter)
 
             div.appendInside(markup(QL1S("a")));
             div.lastChild().setAttribute(QL1S("class"), QL1S("greylink"));
-            div.lastChild().setAttribute(QL1S("href"), QL1S("about:downloads/removeItem?item=") + QString::number(i));
+            div.lastChild().setAttribute(QL1S("href"), QL1S("rekonq:downloads/removeItem?item=") + QString::number(i));
             div.lastChild().setPlainText(i18n("Remove from list"));
 
             break;
@@ -760,7 +769,7 @@ QWebElement NewTabPage::emptyPreview(int index)
             QL1S("file:///") + KIconLoader::global()->iconPath("insert-image", KIconLoader::Desktop));
     prev.findFirst(QL1S("span a")).setPlainText(i18n("Set a Preview..."));
     prev.findFirst(QL1S("a")).setAttribute(QL1S("href"),
-                                           QL1S("about:preview/modify/") + QVariant(index).toString());
+                                           QL1S("rekonq:preview/modify/") + QVariant(index).toString());
 
     setupPreview(prev, index, false);
 
@@ -807,7 +816,7 @@ QWebElement NewTabPage::tabPreview(int winIndex, int tabIndex, const KUrl &url,
     QWebElement prev = markup(QL1S(".thumbnail"));
     QString previewPath = QL1S("file://") + WebSnap::imagePathFromUrl(url);
 
-    QString href = QL1S("about:tabs/show?win=") + QString::number(winIndex) + QL1S("&tab=") + QString::number(tabIndex);
+    QString href = QL1S("rekonq:tabs/show?win=") + QString::number(winIndex) + QL1S("&tab=") + QString::number(tabIndex);
 
     prev.findFirst(QL1S(".preview img")).setAttribute(QL1S("src") , previewPath);
     prev.findFirst(QL1S("a")).setAttribute(QL1S("href"), href);
@@ -832,7 +841,7 @@ QWebElement NewTabPage::closedTabPreview(int index, const KUrl &url, const QStri
                           : IconManager::self()->iconPathForUrl(url)
                           ;
 
-    QString href = QL1S("about:closedTabs/restore?tab=") + QString::number(index);
+    QString href = QL1S("rekonq:closedtabs/restore?tab=") + QString::number(index);
 
     prev.findFirst(QL1S(".preview img")).setAttribute(QL1S("src") , previewPath);
     prev.findFirst(QL1S("a")).setAttribute(QL1S("href"), href);
@@ -856,8 +865,8 @@ void NewTabPage::setupPreview(QWebElement e, int index, bool showControls)
 
     e.findFirst(QL1S(".left")).setAttribute(QL1S("title"), i18n("Reload thumbnail"));
 
-    e.findFirst(QL1S(".left")).setAttribute(QL1S("href"), QL1S("about:preview/reload/") + QVariant(index).toString());
-    e.findFirst(QL1S(".right")).setAttribute(QL1S("href"), QL1S("about:preview/remove/") + QVariant(index).toString());
+    e.findFirst(QL1S(".left")).setAttribute(QL1S("href"), QL1S("rekonq:preview/reload/") + QVariant(index).toString());
+    e.findFirst(QL1S(".right")).setAttribute(QL1S("href"), QL1S("rekonq:preview/remove/") + QVariant(index).toString());
 
     e.setAttribute(QL1S("id"), QL1S("preview") + QVariant(index).toString());
 
@@ -875,7 +884,7 @@ void NewTabPage::setupTabPreview(QWebElement e, int winIndex, int tabIndex)
             QL1S("file:///") + KIconLoader::global()->iconPath("edit-delete", KIconLoader::DefaultState));
     e.findFirst(QL1S(".right")).setAttribute(QL1S("title"), QL1S("Close Tab"));
 
-    QString href = QL1S("about:tabs/remove?win=") + QString::number(winIndex) + QL1S("&tab=") + QString::number(tabIndex);
+    QString href = QL1S("rekonq:tabs/remove?win=") + QString::number(winIndex) + QL1S("&tab=") + QString::number(tabIndex);
     e.findFirst(QL1S(".right")).setAttribute(QL1S("href"), href);
 
     e.setAttribute(QL1S("id"), QL1S("win") + QString::number(winIndex) + QL1S("tab") + QString::number(tabIndex));
@@ -893,7 +902,7 @@ void NewTabPage::removePreview(int index)
     ReKonfig::setPreviewNames(names);
     ReKonfig::setPreviewUrls(urls);
 
-    loadPageForUrl(KUrl("about:favorites"));
+    loadPageForUrl(KUrl("rekonq:favorites"));
 
     ReKonfig::self()->writeConfig();
 }
@@ -906,12 +915,18 @@ void NewTabPage::createBookmarkGroup(const KBookmark &bookmark, QWebElement pare
 
     parent.appendInside(markup(QL1S(".bookmarkfolder")));
     QWebElement folder = parent.lastChild();
-    folder.appendInside(markup(QL1S("h4")));
-    folder.lastChild().setPlainText(group.fullText());
+    folder.appendInside(markup(QL1S("a")));
+    folder.lastChild().setAttribute(QL1S("href"), QL1S("javascript: toggleChildren(\'") + group.fullText() + QL1S("\')"));
+    QWebElement titleElement = folder.lastChild();
+    titleElement.appendInside(markup(QL1S("h4")));
+    titleElement.lastChild().setPlainText(group.fullText());
+
+    folder.appendInside(markup(QL1S("div")));
+    folder.lastChild().setAttribute(QL1S("id"), group.fullText());
 
     while (!bm.isNull())
     {
-        createBookmarkItem(bm, folder);
+        createBookmarkItem(bm, folder.lastChild());
         bm = group.next(bm);
     }
 }
@@ -1011,7 +1026,8 @@ void NewTabPage::initJS()
     dataPath.remove(QL1S("/htmls/home.html"));
 
     includes.replace(QL1S("$DEFAULT_PATH"), dataPath);
-
+    includes.replace(QL1S("$GENERAL_FONT"), QWebSettings::globalSettings()->fontFamily(QWebSettings::StandardFont));
+        
     oldHTML.replace(QL1S("<head>"), includes);
 
     QString javascript;
@@ -1022,7 +1038,7 @@ void NewTabPage::initJS()
     javascript += QL1S("        revert: true,");
     javascript += QL1S("        cursor: \"move\",");
     javascript += QL1S("        distance: 30,");
-    javascript += QL1S("        update: function(event, ui) { window.location.href = \"about:favorites/save\"; }");
+    javascript += QL1S("        update: function(event, ui) { window.location.href = \"rekonq:favorites/save\"; }");
     javascript += QL1S("    });");
     javascript += QL1S("    $( \".thumbnail\" ).disableSelection();");
     javascript += QL1S("});");
@@ -1065,5 +1081,5 @@ void NewTabPage::saveFavorites()
     ReKonfig::setPreviewNames(newNames);
     ReKonfig::setPreviewUrls(newUrls);
 
-    loadPageForUrl(KUrl("about:favorites"));
+    loadPageForUrl(KUrl("rekonq:favorites"));
 }
