@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2010-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -43,19 +43,22 @@ class AppearanceWidget : public QWidget, private Ui::appearance
     Q_OBJECT
 
 public:
-    AppearanceWidget(QWidget *parent = 0);
+    explicit AppearanceWidget(QWidget *parent = 0);
 
     void save();
     bool changed();
     bool isDefault();
-    
-signals:
+
+Q_SIGNALS:
     void changed(bool);
 
-private slots:
+private Q_SLOTS:
     void hasChanged();
-                    
+    void setEncoding(const QString &);
+
 private:
+    void populateEncodingMenu();
+
     bool _changed;
 };
 
