@@ -32,8 +32,8 @@
 #include "rekonq.h"
 
 // Config Includes
-#include "config-qca2.h"
-#include "config-qtoauth.h"
+#include <config-qca2.h>
+#include <config-qtoauth.h>
 
 // Local Includes
 #include "application.h"
@@ -49,12 +49,28 @@
 #endif
 
 // KDE Includes
-#include <klocalizedstring.h>
+#include <KLocalizedString>
 
 // Qt Includes
 #include <QPointer>
 
 
+QWeakPointer<SyncManager> SyncManager::s_syncManager;
+
+
+SyncManager *SyncManager::self()
+{
+    if (s_syncManager.isNull())
+    {
+        s_syncManager = new SyncManager(qApp);
+    }
+    return s_syncManager.data();
+}
+
+
+// ----------------------------------------------------------------------------------------------
+
+
 SyncManager::SyncManager(QObject *parent)
     : QObject(parent)
 {
@@ -106,14 +122,14 @@ void SyncManager::loadSettings()
 
         // bookmarks
         ReKonfig::syncBookmarks()
-        ? connect(rApp->bookmarkManager(), SIGNAL(bookmarksUpdated()), this, SLOT(syncBookmarks()))
-        : disconnect(rApp->bookmarkManager(), SIGNAL(bookmarksUpdated()), this, SLOT(syncBookmarks()))
+        ? connect(BookmarkManager::self(), SIGNAL(bookmarksUpdated()), this, SLOT(syncBookmarks()))
+        : disconnect(BookmarkManager::self(), SIGNAL(bookmarksUpdated()), this, SLOT(syncBookmarks()))
         ;
 
         // history
         ReKonfig::syncHistory()
-        ? connect(rApp->historyManager(), SIGNAL(historySaved()), this, SLOT(syncHistory()))
-        : disconnect(rApp->historyManager(), SIGNAL(historySaved()), this, SLOT(syncHistory()))
+        ? connect(HistoryManager::self(), SIGNAL(historySaved()), this, SLOT(syncHistory()))
+        : disconnect(HistoryManager::self(), SIGNAL(historySaved()), this, SLOT(syncHistory()))
         ;
 
         _syncImplementation.data()->initialLoadAndCheck();
@@ -122,10 +138,10 @@ void SyncManager::loadSettings()
     else
     {
         // bookmarks
-        disconnect(rApp->bookmarkManager(), SIGNAL(bookmarksUpdated()), this, SLOT(syncBookmarks()));
+        disconnect(BookmarkManager::self(), SIGNAL(bookmarksUpdated()), this, SLOT(syncBookmarks()));
 
         // history
-        disconnect(rApp->historyManager(), SIGNAL(historySaved()), this, SLOT(syncHistory()));
+        disconnect(HistoryManager::self(), SIGNAL(historySaved()), this, SLOT(syncHistory()));
     }
 }
 
