@@ -44,7 +44,6 @@
 
 #include "adblockwidget.h"
 #include "bookmarkwidget.h"
-#include "favoritewidget.h"
 #include "rsswidget.h"
 
 #include "completionwidget.h"
@@ -152,8 +151,7 @@ UrlBar::UrlBar(QWidget *parent)
 
     connect(_tab, SIGNAL(urlChanged(QUrl)), this, SLOT(setQUrl(QUrl)));
     connect(_tab, SIGNAL(loadFinished(bool)), this, SLOT(loadFinished()));
-    connect(_tab, SIGNAL(loadStarted()), this, SLOT(clearRightIcons()));
-    connect(_tab, SIGNAL(iconChanged()), this, SLOT(refreshFavicon()));
+    connect(_tab, SIGNAL(loadStarted()), this, SLOT(loadStarted()));
 
     // bookmark icon
     connect(BookmarkManager::self(), SIGNAL(bookmarksUpdated()), this, SLOT(updateRightIcons()));
@@ -178,20 +176,43 @@ UrlBar::~UrlBar()
 
 void UrlBar::setQUrl(const QUrl& url)
 {
-    if (url.scheme() == QL1S("about"))
+    if (url.scheme() == QL1S("rekonq"))
         return;
 
+    // we don't set empty url
+    if (url.isEmpty())
+        return;
+    
     clearFocus();
-    KLineEdit::setUrl(url);
+    
+    // Workaround for KLineEdit bug: incorrectly displaying
+    // unicode symbols at query parameter
+    const QByteArray urlTextData = url.toString().toUtf8();
+    const QString humanReadableUrl = QString::fromUtf8(
+        QByteArray::fromPercentEncoding(urlTextData).constData()
+    );
+    
+    // End workaround
+    setText(humanReadableUrl);
+
     setCursorPosition(0);
-    refreshFavicon();
 }
 
 
 void UrlBar::loadRequestedUrl(const KUrl& url, Rekonq::OpenType type)
 {
     clearFocus();
-    setUrl(url);
+    
+    // Workaround for KLineEdit bug: incorrectly displaying
+    // unicode symbols at query parameter
+    const QByteArray urlTextData = url.prettyUrl().toUtf8();
+    const QString humanReadableUrl = QString::fromUtf8(
+        QByteArray::fromPercentEncoding(urlTextData).constData()
+    );
+    
+    // End workaround
+    setText(humanReadableUrl);
+    
     rApp->loadUrl(url, type);
 }
 
@@ -238,14 +259,6 @@ void UrlBar::paintEvent(QPaintEvent *event)
     int progr = _tab->progress();
     if (progr == 0 || progr == 100)
     {
-        if (_tab->url().scheme() == QL1S("https"))
-        {
-            backgroundColor = _tab->page()->hasSslValid()
-                              ? colorScheme.background(KColorScheme::PositiveBackground).color()
-                              : colorScheme.background(KColorScheme::NegativeBackground).color();
-
-            foregroundColor = colorScheme.foreground(KColorScheme::NormalText).color();
-        }
         p.setBrush(QPalette::Base, backgroundColor);
         p.setBrush(QPalette::Text, foregroundColor);
     }
@@ -295,34 +308,41 @@ void UrlBar::paintEvent(QPaintEvent *event)
 
 void UrlBar::keyPressEvent(QKeyEvent *event)
 {
-    QString currentText = text().trimmed();
+    QString trimmedText = text().trimmed();
 
-    if (currentText.isEmpty())
+    if (trimmedText.isEmpty())
+    {
+        disconnect(_icon);
+        _icon->setIcon(KIcon("arrow-right"));
         return KLineEdit::keyPressEvent(event);
-
+    }
+    
     // this handles the Modifiers + Return key combinations
     if (event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter)
     {
+        KUrl urlToLoad;
         switch (event->modifiers())
         {
         case Qt::AltModifier:
-            loadRequestedUrl(currentText, Rekonq::NewFocusedTab);
+            urlToLoad = UrlResolver::urlFromTextTyped(text());
+            loadRequestedUrl(urlToLoad, Rekonq::NewFocusedTab);
             break;
 
         case Qt::ControlModifier:
-            loadRequestedUrl(guessUrlWithCustomFirstLevel(currentText, QL1S(".com")));
+            loadRequestedUrl(guessUrlWithCustomFirstLevel(trimmedText, QL1S(".com")));
             break;
 
         case 0x06000000: // Qt::ControlModifier | Qt::ShiftModifier:
-            loadRequestedUrl(guessUrlWithCustomFirstLevel(currentText, QL1S(".org")));
+            loadRequestedUrl(guessUrlWithCustomFirstLevel(trimmedText, QL1S(".org")));
             break;
 
         case Qt::ShiftModifier:
-            loadRequestedUrl(guessUrlWithCustomFirstLevel(currentText, QL1S(".net")));
+            loadRequestedUrl(guessUrlWithCustomFirstLevel(trimmedText, QL1S(".net")));
             break;
 
         default:
-            loadRequestedUrl(currentText);
+            urlToLoad = UrlResolver::urlFromTextTyped(text());
+            loadRequestedUrl(urlToLoad);
             break;
         }
     }
@@ -330,7 +350,7 @@ void UrlBar::keyPressEvent(QKeyEvent *event)
     if (event->key() == Qt::Key_Escape)
     {
         clearFocus();
-        if (!(_tab->url().protocol() == QL1S("about")))
+        if (!(_tab->url().protocol() == QL1S("rekonq")))
             setText(_tab->url().prettyUrl());
         event->accept();
     }
@@ -367,21 +387,36 @@ void UrlBar::dropEvent(QDropEvent *event)
 }
 
 
+void UrlBar::loadStarted()
+{
+    _icon->setIcon(KIcon("text-html"));
+    clearRightIcons();
+}
+
+
 void UrlBar::loadFinished()
 {
-    if (_tab->url().scheme() == QL1S("about"))
+    refreshFavicon();
+    updateRightIcons();
+}
+
+
+void UrlBar::updateRightIcons()
+{
+    if (_tab->isPageLoading())
+        return;
+
+    clearRightIcons();
+    
+    if (_tab->url().scheme() == QL1S("rekonq"))
     {
         update();
         return;
     }
-
+    
     // show bookmark info
     IconButton *bt = addRightIcon(UrlBar::BK);
-    connect(bt, SIGNAL(clicked(QPoint)), this, SLOT(manageBookmarks()));
-
-    // show favorite icon
-    IconButton *fbt = addRightIcon(UrlBar::Favorite);
-    connect(fbt, SIGNAL(clicked(QPoint)), this, SLOT(manageFavorites(QPoint)));
+    connect(bt, SIGNAL(clicked(QPoint)), this, SLOT(manageStarred(QPoint)));
 
     // show KGet downloads??
     if (!KStandardDirs::findExe("kget").isNull() && ReKonfig::kgetList())
@@ -397,14 +432,6 @@ void UrlBar::loadFinished()
         connect(bt, SIGNAL(clicked(QPoint)), this, SLOT(showRSSInfo(QPoint)));
     }
 
-    // show SSL
-    if (_tab->url().scheme() == QL1S("https"))
-    {
-        // NOTE: the choice for the right SSL icon is done in the addRightIcon method
-        IconButton *bt = addRightIcon(UrlBar::SSL);
-        connect(bt, SIGNAL(clicked(QPoint)), _tab->page(), SLOT(showSSLInfo(QPoint)));
-    }
-
     // Show adblock
     if (AdBlockManager::self()->isEnabled())
     {
@@ -421,16 +448,6 @@ void UrlBar::loadFinished()
 }
 
 
-void UrlBar::updateRightIcons()
-{
-    if (!_tab->isPageLoading())
-    {
-        clearRightIcons();
-        loadFinished();
-    }
-}
-
-
 void UrlBar::mouseDoubleClickEvent(QMouseEvent *event)
 {
     Q_UNUSED(event);
@@ -526,22 +543,15 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         rightIcon->setIcon(KIcon("application-rss+xml"));
         rightIcon->setToolTip(i18n("List all available RSS feeds"));
         break;
-    case UrlBar::SSL:
-        _tab->page()->hasSslValid()
-        ? rightIcon->setIcon(KIcon("object-locked"))
-        : rightIcon->setIcon(KIcon("object-unlocked"));
-        rightIcon->setToolTip(i18n("Show SSL Info"));
-        break;
     case UrlBar::BK:
-        if (BookmarkManager::self()->bookmarkForUrl(_tab->url()).isNull())
+        if (BookmarkManager::self()->bookmarkForUrl(_tab->url()).isNull() && 
+            !ReKonfig::previewUrls().contains(_tab->url().url()))
         {
             rightIcon->setIcon(KIcon("bookmarks").pixmap(32, 32, QIcon::Disabled));
-            rightIcon->setToolTip(i18n("Bookmark this page"));
         }
         else
         {
             rightIcon->setIcon(KIcon("bookmarks"));
-            rightIcon->setToolTip(i18n("Edit this bookmark"));
         }
         break;
     case UrlBar::SearchEngine:
@@ -555,18 +565,6 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         rightIcon->setToolTip(i18n("Add search engine"));
         break;
     }
-    case UrlBar::Favorite:
-        if (ReKonfig::previewUrls().contains(_tab->url().url()))
-        {
-            rightIcon->setIcon(KIcon("emblem-favorite"));
-            rightIcon->setToolTip(i18n("Remove from favorite"));
-        }
-        else
-        {
-            rightIcon->setIcon(KIcon("emblem-favorite").pixmap(32, 32, QIcon::Disabled));
-            rightIcon->setToolTip(i18n("Add to favorites"));
-        }
-        break;
     case UrlBar::AdBlock:
     {
         QStringList hosts = ReKonfig::whiteReferer();
@@ -578,7 +576,7 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         else
         {
             rightIcon->setIcon(KIcon("preferences-web-browser-adblock").pixmap(32, 32, QIcon::Disabled));
-            rightIcon->setToolTip(i18n("AdBlock is NOT enabled on this site"));
+            rightIcon->setToolTip(i18n("AdBlock is not enabled on this site"));
         }
         break;
     }
@@ -627,6 +625,7 @@ void UrlBar::detectTypedString(const QString &typed)
 {
     if (typed.count() == 1)
     {
+        _icon->setIcon(KIcon("arrow-right"));
         QTimer::singleShot(0, this, SLOT(suggest()));
         return;
     }
@@ -648,20 +647,38 @@ void UrlBar::suggest()
 
 void UrlBar::refreshFavicon()
 {
+    disconnect(_icon);
+    
+    const QString scheme = _tab->url().protocol();
+    
     if (_tab->page()->settings()->testAttribute(QWebSettings::PrivateBrowsingEnabled))
     {
         _icon->setIcon(KIcon("view-media-artist"));
         return;
     }
+    
+    if (scheme == QL1S("https"))
+    {
+        if (_tab->page()->hasSslValid())
+        {
+            _icon->setIcon(KIcon("security-high"));
+        }
+        else
+        {
+            _icon->setIcon(KIcon("security-low"));
+        }
+        
+        connect(_icon, SIGNAL(clicked(QPoint)), _tab->page(), SLOT(showSSLInfo(QPoint)));
+        return;
+    }
 
-    KUrl u = _tab->url();
-    if (u.scheme() == QL1S("about"))
+    if (scheme == QL1S("rekonq"))
     {
         _icon->setIcon(KIcon("arrow-right"));
         return;
     }
 
-    _icon->setIcon(IconManager::self()->iconForUrl(u));
+    _icon->setIcon(KIcon("text-html"));
 }
 
 
@@ -687,7 +704,7 @@ void UrlBar::delSlot()
 
 void UrlBar::manageBookmarks()
 {
-    if (_tab->url().scheme() == QL1S("about"))
+    if (_tab->url().scheme() == QL1S("rekonq"))
         return;
 
     KBookmark bookmark = BookmarkManager::self()->bookmarkForUrl(_tab->url());
@@ -713,48 +730,13 @@ void UrlBar::manageBookmarks()
 }
 
 
-void UrlBar::manageFavorites(QPoint pos)
-{
-    IconButton *bt = qobject_cast<IconButton *>(this->sender());
-    if (!bt)
-        return;
-
-    if (_tab->url().scheme() == QL1S("about"))
-        return;
-
-    if (ReKonfig::previewUrls().contains(_tab->url().url()))
-    {
-        // remove site from favorites
-        FavoriteWidget *widget = new FavoriteWidget(_tab, window());
-        connect(widget, SIGNAL(updateIcon()), this, SLOT(updateRightIcons()));
-        widget->showAt(pos);
-        return;
-    }
-
-    // else, add as favorite
-    QStringList urls = ReKonfig::previewUrls();
-    urls << _tab->url().url();
-    ReKonfig::setPreviewUrls(urls);
-
-    QStringList titles = ReKonfig::previewNames();
-    titles << _tab->view()->title();
-    ReKonfig::setPreviewNames(titles);
-
-    // also, save a site snapshot
-    WebSnap *snap = new WebSnap(_tab->url(), this);
-    Q_UNUSED(snap);
-
-    updateRightIcons();
-}
-
-
 void UrlBar::manageAdBlock(QPoint pos)
 {
     IconButton *bt = qobject_cast<IconButton *>(this->sender());
     if (!bt)
         return;
 
-    if (_tab->url().scheme() == QL1S("about"))
+    if (_tab->url().scheme() == QL1S("rekonq"))
         return;
 
     AdBlockWidget *widget = new AdBlockWidget(_tab->url(), this);
@@ -809,3 +791,79 @@ void UrlBar::showRSSInfo(const QPoint &pos)
     RSSWidget *widget = new RSSWidget(map, window());
     widget->showAt(pos);
 }
+
+
+void UrlBar::manageStarred(QPoint pos)
+{
+    KMenu menu;
+    KAction *a;
+
+    // Bookmarks
+    if (BookmarkManager::self()->bookmarkForUrl(_tab->url()).isNull())
+    {
+        a = new KAction(KIcon(KIcon("bookmarks").pixmap(32, 32, QIcon::Disabled)), i18n("Add Bookmark"), &menu);
+        connect(a, SIGNAL(triggered(bool)), this, SLOT(manageBookmarks()));
+    }
+    else
+    {
+        a = new KAction(KIcon("bookmarks"), i18n("Edit Bookmark"), &menu);
+        connect(a, SIGNAL(triggered(bool)), this, SLOT(manageBookmarks()));        
+    }
+    menu.addAction(a);
+    
+    // Favorites
+    if (ReKonfig::previewUrls().contains(_tab->url().url()))
+    {
+        a = new KAction(KIcon("emblem-favorite"), i18n("Remove from Favorites"), &menu);
+        connect(a, SIGNAL(triggered(bool)), this, SLOT(removeFromFavorites()));        
+    }
+    else
+    {
+        a = new KAction(KIcon(KIcon("emblem-favorite").pixmap(32, 32, QIcon::Disabled)), i18n("Add to Favorites"), &menu);
+        connect(a, SIGNAL(triggered(bool)), this, SLOT(addToFavorites()));
+    }
+    menu.addAction(a);
+    
+    QPoint p(pos.x() - menu.sizeHint().width() + 15, pos.y() + 15);
+    menu.exec(p);
+}
+
+
+void UrlBar::addToFavorites()
+{
+    if (_tab->url().scheme() == QL1S("rekonq"))
+        return;
+
+    // else, add as favorite
+    QStringList urls = ReKonfig::previewUrls();
+    urls << _tab->url().url();
+    ReKonfig::setPreviewUrls(urls);
+
+    QStringList titles = ReKonfig::previewNames();
+    titles << _tab->view()->title();
+    ReKonfig::setPreviewNames(titles);
+
+    // also, save a site snapshot
+    WebSnap *snap = new WebSnap(_tab->url(), this);
+    Q_UNUSED(snap);
+
+    updateRightIcons();
+}
+
+
+void UrlBar::removeFromFavorites()
+{
+    if (_tab->url().scheme() == QL1S("rekonq"))
+        return;
+
+    QStringList urls = ReKonfig::previewUrls();
+    if (urls.removeOne(_tab->url().url()))
+    {
+        ReKonfig::setPreviewUrls(urls);
+        QStringList titles = ReKonfig::previewNames();
+        titles.removeOne(_tab->view()->title());
+        ReKonfig::setPreviewNames(titles);
+
+        updateRightIcons();
+    }    
+}
