@@ -80,10 +80,8 @@ public:
     {
         KGet         = 0x00000001,
         RSS          = 0x00000010,
-        SSL          = 0x00000100,
         BK           = 0x00001000,
         SearchEngine = 0x00010000,
-        Favorite     = 0x00100000,
         AdBlock      = 0x01000000
     };
 
@@ -102,6 +100,7 @@ public Q_SLOTS:
 private Q_SLOTS:
     void loadRequestedUrl(const KUrl& url, Rekonq::OpenType = Rekonq::CurrentTab);
 
+    void loadStarted();
     void loadFinished();
 
     void clearRightIcons();
@@ -110,8 +109,11 @@ private Q_SLOTS:
     void detectTypedString(const QString &);
     void suggest();
 
+    void manageStarred(QPoint);
     void manageAdBlock(QPoint);
-    void manageFavorites(QPoint);
+
+    void addToFavorites();
+    void removeFromFavorites();
 
     void refreshFavicon();
 
