@@ -64,6 +64,8 @@
 #include <KWebWallet>
 #include <KProtocolInfo>
 #include <KRun>
+#include <KMimeType>
+#include <KProtocolManager>
 
 #include <KIO/Job>
 #include <KIO/JobUiDelegate>
@@ -125,7 +127,7 @@ static void extractMimeType(const QNetworkReply* reply, QString& mimeType)
 // ---------------------------------------------------------------------------------
 
 
-WebPage::WebPage(QWidget *parent, bool isPrivateBrowsing)
+WebPage::WebPage(bool isPrivateBrowsing, QWidget *parent)
     : KWebPage(parent, KWalletIntegration)
     , _networkAnalyzer(false)
     , _isOnRekonqPage(false)
@@ -141,7 +143,7 @@ WebPage::WebPage(QWidget *parent, bool isPrivateBrowsing)
         // The problem is basically cookie related. This way we lose some features in private
         // browsing mode, but we ensure PRIVACY! This change cannot be reverted until a proper
         // fix for KIO (or the right workaround for rekonq) will be found.
-        QNetworkAccessManager *manager = new QNetworkAccessManager(this);
+        QNetworkAccessManager *manager = NetworkAccessManager::privateAccessManager();
         setNetworkAccessManager(manager);
 
         // ----- last stuffs
@@ -176,7 +178,7 @@ WebPage::WebPage(QWidget *parent, bool isPrivateBrowsing)
     connect(this, SIGNAL(loadStarted()), this, SLOT(loadStarted()));
     connect(this, SIGNAL(loadFinished(bool)), this, SLOT(loadFinished(bool)));
 
-    connect(this, SIGNAL(frameCreated(QWebFrame *)), AdBlockManager::self(), SLOT(applyHidingRules(QWebFrame *)));
+    connect(this, SIGNAL(frameCreated(QWebFrame*)), AdBlockManager::self(), SLOT(applyHidingRules(QWebFrame*)));
     
     // protocol handler signals
     connect(&_protHandler, SIGNAL(downloadUrl(KUrl)), this, SLOT(downloadUrl(KUrl)));
@@ -241,12 +243,19 @@ bool WebPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &r
         tab->setPart(0, KUrl());     // re-enable the view page
     }
 
-    // reset webpage values
-    _suggestedFileName.clear();
-    _loadingUrl = request.url();
-
     const bool isMainFrameRequest = (frame == mainFrame());
+    if (isMainFrameRequest)
+    {
+        // reset webpage values
+        _suggestedFileName.clear();
+        _loadingUrl = request.url();
 
+        // Set Page URL: This is needed for 2 reasons.
+        // 1) WebKit is slot setting url in some case. Having an initial URL set seems snappier ;)
+        // 2) When WebKit cannot set URL (eg: network down), urlbar URL is NOT set
+        emit initialUrl(_loadingUrl);        
+    }
+    
     if (frame)
     {
         if (_protHandler.preHandling(request, frame))
@@ -329,12 +338,81 @@ WebPage *WebPage::createWindow(QWebPage::WebWindowType type)
     if (type == QWebPage::WebModalDialog)
         kDebug() << "Modal Dialog";
 
-    WebPage* p = new WebPage;
+    bool isPrivateBrowsing = settings()->testAttribute(QWebSettings::PrivateBrowsingEnabled);
+
+    // private page links open private pages. See BUG: 320218
+    WebPage* p = new WebPage(isPrivateBrowsing);
     emit pageCreated(p);
     return p;
 }
 
 
+// From Konqueror's konqsettings.cpp
+static bool alwaysEmbedMimeTypeGroup(const QString &mimeType)
+{
+    if (mimeType.startsWith("inode") || mimeType.startsWith("Browser"))
+        return true;
+    return false;
+}
+
+
+// From Konqueror's konqsettings.cpp.
+// config and m_embedMap were manually added to make the function static.
+// TODO: Probably should be moved into some library for KDE Frameworks...
+static bool shouldEmbed(const QString &_mimeType)
+{
+    KSharedConfig::Ptr config = KSharedConfig::openConfig("filetypesrc", KConfig::NoGlobals);
+    QMap<QString, QString> m_embedMap = config->entryMap("EmbedSettings");
+
+    KMimeType::Ptr mime = KMimeType::mimeType(_mimeType, KMimeType::ResolveAliases);
+    if (!mime) {
+        kWarning() << "Unknown mimetype" << _mimeType;
+        return false; // unknown mimetype!
+    }
+    const QString mimeType = mime->name();
+
+    // First check in user's settings whether to embed or not
+    // 1 - in the filetypesrc config file (written by the configuration module)
+    QMap<QString, QString>::const_iterator it = m_embedMap.constFind( QString::fromLatin1("embed-")+mimeType );
+    if ( it != m_embedMap.constEnd() ) {
+        kDebug() << mimeType << it.value();
+        return it.value() == QLatin1String("true");
+    }
+    // 2 - in the configuration for the group if nothing was found in the mimetype
+    if (alwaysEmbedMimeTypeGroup(mimeType))
+        return true; //always embed mimetype inode/*, Browser/* and Konqueror/*
+    const QString mimeTypeGroup = mimeType.left(mimeType.indexOf('/'));
+    it = m_embedMap.constFind( QString::fromLatin1("embed-")+mimeTypeGroup );
+    if ( it != m_embedMap.constEnd() ) {
+        kDebug() << mimeType << "group setting:" << it.value();
+        return it.value() == QLatin1String("true");
+    }
+    // 2 bis - configuration for group of parent mimetype, if different
+    if (mimeType[0].isLower()) {
+        QStringList parents;
+        parents.append(mimeType);
+        while (!parents.isEmpty()) {
+            const QString parent = parents.takeFirst();
+            if (alwaysEmbedMimeTypeGroup(parent)) {
+                return true;
+            }
+            KMimeType::Ptr mime = KMimeType::mimeType(parent);
+            Q_ASSERT(mime); // how could the -parent- be null?
+            if (mime)
+                parents += mime->parentMimeTypes();
+        }
+    }
+
+    // 3 - if no config found, use default.
+    // Note: if you change those defaults, also change keditfiletype/mimetypedata.cpp !
+    // Embedding is false by default except for image/* and for zip, tar etc.
+    const bool hasLocalProtocolRedirect = !KProtocolManager::protocolForArchiveMimetype(mimeType).isEmpty();
+    if (mimeTypeGroup == "image" || mimeTypeGroup == "multipart" || hasLocalProtocolRedirect)
+        return true;
+    return false;
+}
+
+
 void WebPage::handleUnsupportedContent(QNetworkReply *reply)
 {
     Q_ASSERT(reply);
@@ -472,7 +550,9 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
         webAppMode = true;
 
     // case KParts::BrowserRun::Embed
-    KParts::ReadOnlyPart *pa = KMimeTypeTrader::createPartInstanceFromQuery<KParts::ReadOnlyPart>(_mimeType, view(), this, QString());
+    KParts::ReadOnlyPart *pa = 0;
+    if (shouldEmbed(_mimeType)) // Only create the KPart if we are actually supposed to embed.
+        pa = KMimeTypeTrader::createPartInstanceFromQuery<KParts::ReadOnlyPart>(_mimeType, view(), this, QString());
     if (pa && !webAppMode)
     {
         _isOnRekonqPage = true;
@@ -513,9 +593,6 @@ void WebPage::loadFinished(bool ok)
 {
     Q_UNUSED(ok);
 
-    // Provide site icon. Can this be moved to loadStarted??
-    IconManager::self()->provideIcon(mainFrame(), _loadingUrl);
-
     // KWallet Integration
     QStringList list = ReKonfig::walletBlackList();
     if (wallet()
@@ -629,7 +706,7 @@ QString WebPage::errorPage(QNetworkReply *reply)
     // test to see if networking is enabled on the system
     if (Solid::Networking::status() != Solid::Networking::Connected)
     {
-        msg += QL1S("<h2>") + i18n("Network is NOT available") + QL1S("</h2>");
+        msg += QL1S("<h2>") + i18n("Network is not available") + QL1S("</h2>");
 
         QString faceIconPath = QString("file://") + KIconLoader::global()->iconPath("face-surprise" , -KIconLoader::SizeHuge, false);
         msg += QL1S("<table>");
@@ -639,7 +716,7 @@ QString WebPage::errorPage(QNetworkReply *reply)
 
         msg += QL1S("<p>");
 
-        msg += i18n("Maybe you are having problems with your network settings.<br />Try checking your <a href=\"%1\">network connections</a>, your <a href=\"%2\">proxy settings</a> and your <a href=\"%3\">firewall</a>.<br /><br />Then <a href=\"%4\">try again</a>.<br />", QL1S("about:settings/network"), QL1S("about:settings/proxy"), QL1S("about:settings/firewall"), urlString);
+        msg += i18n("Maybe you are having problems with your network settings.<br />Try checking your <a href=\"%1\">network connections</a>, your <a href=\"%2\">proxy settings</a> and your <a href=\"%3\">firewall</a>.<br /><br />Then <a href=\"%4\">try again</a>.<br />", QL1S("rekonq:settings/network"), QL1S("rekonq:settings/proxy"), QL1S("rekonq:settings/firewall"), urlString);
 
         msg += QL1S("</p>");
 
@@ -651,11 +728,44 @@ QString WebPage::errorPage(QNetworkReply *reply)
         html.replace(QL1S("$DEFAULT_PATH"), dataPath);
         html.replace(QL1S("$PAGE_TITLE"), title);
         html.replace(QL1S("$MAIN_CONTENT"), msg);
-
+        html.replace(QL1S("$GENERAL_FONT"), QWebSettings::globalSettings()->fontFamily(QWebSettings::StandardFont));
+        
         return html;
     }
 
+    QString errString = reply->errorString().toLower();
+    if (errString.contains(QL1S("proxy")))
+    {
+        msg += QL1S("<h2>") + i18n("Oops... Proxy problems!") + QL1S("</h2>");
 
+        QString faceIconPath = QString("file://") + KIconLoader::global()->iconPath("face-surprise" , -KIconLoader::SizeHuge, false);
+        msg += QL1S("<table>");
+        msg += QL1S("<tr><td width=\"100px\">");
+        msg += QL1S("<img style=\"margin: 0 auto;\" src=\"") + faceIconPath + QL1S("\" />");
+        msg += QL1S("</td><td>");
+
+        msg += QL1S("<p><em>") + reply->errorString() + QL1S("</em></p>");
+        
+        msg += QL1S("<p>");
+
+        msg += i18n("It seems you are having problems with your <a href=\"%1\">proxy settings</a>. Try checking them, <br /><br />then <a href=\"%2\">try again</a>.<br />", QL1S("rekonq:settings/proxy"), urlString);
+
+        msg += QL1S("</p>");
+
+        msg += QL1S("</td></tr></table>");
+
+        // done. Replace variables and show it
+        QString html = QL1S(file.readAll());
+
+        html.replace(QL1S("$DEFAULT_PATH"), dataPath);
+        html.replace(QL1S("$PAGE_TITLE"), title);
+        html.replace(QL1S("$MAIN_CONTENT"), msg);
+        html.replace(QL1S("$GENERAL_FONT"), QWebSettings::globalSettings()->fontFamily(QWebSettings::StandardFont));
+        
+        return html;
+    }
+    
+    // general error page
     msg += QL1S("<h2>") + i18n("Oops! Cannot load <em>%1</em>", urlString) + QL1S("</h1>");
 
     QString faceIconPath = QString("file://") + KIconLoader::global()->iconPath("face-surprise" , -KIconLoader::SizeHuge, false);
@@ -672,23 +782,25 @@ QString WebPage::errorPage(QNetworkReply *reply)
     msg += QL1S("<p>");
     if (defaultEngine)
     {
-        msg += i18n("Ask your default search engine about:");
-        msg += QL1S(" <a href=\"") + SearchEngine::buildQuery(defaultEngine, urlString) + QL1S("\">");
-        msg += i18n("search with %1", defaultEngine->name());
-        msg += QL1S("</a>!<br />");
+        msg += i18n("Ask your default search engine about: "
+                    "<a href=\"%1\">search with %2</a>.<br />",
+                    SearchEngine::buildQuery(defaultEngine, urlString),
+                    defaultEngine->name());
     }
     else
     {
-        msg += i18n("You don't have a default search engine set. We won't suggest you one.");
+        msg += i18n("You do not have a default search engine set. We will not suggest you one.");
     }
     msg += QL1S("</p>");
 
     msg += QL1S("<p>");
-    msg += i18n("Consult a cached snapshot of the site: ");
-    msg += i18n("try checking the <a href=\"%1\">Wayback Machine</a>", QL1S("http://wayback.archive.org/web/*/") + urlString);
-    msg += i18n(" or the <a href=\"%1\">Google Cache</a>.", QL1S("http://google.com/search?q=cache:") + urlString);
+    msg += i18n("Consult a cached snapshot of the site: "
+                "try checking the <a href=\"%1\">Wayback Machine</a>"
+                " or the <a href=\"%2\">Google Cache</a>.",
+                QL1S("http://wayback.archive.org/web/*/") + urlString,
+                QL1S("http://google.com/search?q=cache:") + urlString);
     msg += QL1S("</p>");
-
+    msg += QL1S("<h5>") + i18n("<a href='%1'>Try Again</a>", urlString) + QL1S("</h5>");
     msg += QL1S("</td></tr></table>");
 
     // done. Replace variables and show it
@@ -697,7 +809,8 @@ QString WebPage::errorPage(QNetworkReply *reply)
     html.replace(QL1S("$DEFAULT_PATH"), dataPath);
     html.replace(QL1S("$PAGE_TITLE"), title);
     html.replace(QL1S("$MAIN_CONTENT"), msg);
-
+    html.replace(QL1S("$GENERAL_FONT"), QWebSettings::globalSettings()->fontFamily(QWebSettings::StandardFont));
+        
     return html;
 }
 
