@@ -58,6 +58,10 @@
 #include <KParts/Part>
 #include <KParts/BrowserExtension>
 
+#ifdef HAVE_KACTIVITIES
+#include <KActivities/ResourceInstance>
+#endif
+
 // Qt Includes
 #include <QVBoxLayout>
 #include <QPrintDialog>
@@ -74,7 +78,11 @@ WebTab::WebTab(QWidget *parent, bool isPrivateBrowsing)
     , m_part(0)
     , m_zoomFactor(10)
     , m_isPrivateBrowsing(isPrivateBrowsing)
+    , m_isWebApp(false)
     , m_splitter(new QSplitter(this))
+#ifdef HAVE_KACTIVITIES
+    , m_activityResourceInstance(0)
+#endif
 {
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
@@ -104,24 +112,38 @@ WebTab::WebTab(QWidget *parent, bool isPrivateBrowsing)
 
     // Connect webview signals with related webtab ones
     connect(view(), SIGNAL(loadFinished(bool)),     this, SIGNAL(loadFinished(bool)));
-    connect(view(), SIGNAL(loadProgress(int)),      this, SIGNAL(loadProgress (int)));
+    connect(view(), SIGNAL(loadProgress(int)),      this, SIGNAL(loadProgress(int)));
     connect(view(), SIGNAL(loadStarted()),          this, SIGNAL(loadStarted()));
     connect(view(), SIGNAL(urlChanged(QUrl)),       this, SIGNAL(urlChanged(QUrl)));
     connect(view(), SIGNAL(titleChanged(QString)),  this, SIGNAL(titleChanged(QString)));
     connect(view(), SIGNAL(iconChanged()),          this, SIGNAL(iconChanged()));
+    
+    connect(page(), SIGNAL(initialUrl(QUrl)),       this, SIGNAL(urlChanged(QUrl)));
 
     if (!parent)
     {
+        m_isWebApp = true;
         connect(this, SIGNAL(titleChanged(QString)), this, SLOT(webAppTitleChanged(QString)));
         connect(this, SIGNAL(iconChanged()), this, SLOT(webAppIconChanged()));
     }
 
-    connect(view(), SIGNAL(loadProgress(int)), this, SLOT(updateProgress(int)));
-    connect(view(), SIGNAL(loadStarted()), this, SLOT(resetProgress()));
+    connect(view(), SIGNAL(loadProgress(int)),  this, SLOT(updateProgress(int)));
+    connect(view(), SIGNAL(loadStarted()),      this, SLOT(resetProgress()));
     connect(view(), SIGNAL(loadFinished(bool)), this, SLOT(loadFinished()));
-
+    connect(view(), SIGNAL(zoomChanged(int)),   this, SLOT(setZoom(int)));
+    
     // Session Manager
     connect(view(), SIGNAL(loadFinished(bool)), SessionManager::self(), SLOT(saveSession()));
+    
+#ifdef HAVE_KACTIVITIES
+    if (m_isPrivateBrowsing)
+        return;
+    
+    m_activityResourceInstance = new KActivities::ResourceInstance(window()->winId(), this);
+    
+    connect(this, SIGNAL(urlChanged(QUrl)), m_activityResourceInstance, SLOT(setUri(QUrl)));
+    connect(this, SIGNAL(titleChanged(QString)), m_activityResourceInstance, SLOT(setTitle(QString)));
+#endif
 }
 
 
@@ -178,7 +200,7 @@ KUrl WebTab::url()
 
 QString WebTab::title()
 {
-    if (view() && url().protocol() == QL1S("about"))
+    if (view() && url().protocol() == QL1S("rekonq"))
         return view()->title();
     
     if (page() && page()->isOnRekonqPage())
@@ -404,7 +426,7 @@ void WebTab::zoomIn()
 {
     if (m_zoomFactor == 50)
     {
-        emit infoToShow(i18n("Max zoom reached: ") + QString::number(m_zoomFactor * 10) + QL1S("%"));
+        emit infoToShow(i18n("Maximum zoom reached: %1%", QString::number(m_zoomFactor * 10)));
         return;
     }
 
@@ -415,7 +437,7 @@ void WebTab::zoomIn()
 
     view()->setZoomFactor(QVariant(m_zoomFactor).toReal() / 10);
 
-    emit infoToShow(i18n("Zooming: ") + QString::number(m_zoomFactor * 10) + QL1S("%"));
+    setZoom(m_zoomFactor);
 }
 
 
@@ -423,14 +445,14 @@ void WebTab::zoomOut()
 {
     if (m_zoomFactor == 1)
     {
-        emit infoToShow(i18n("Min zoom reached: ") + QString::number(m_zoomFactor * 10) + QL1S("%"));
+        emit infoToShow(i18n("Minimum zoom reached: %1%", QString::number(m_zoomFactor * 10)));
         return;
     }
 
     m_zoomFactor--;
     view()->setZoomFactor(QVariant(m_zoomFactor).toReal() / 10);
 
-    emit infoToShow(i18n("Zooming: ") + QString::number(m_zoomFactor * 10) + QL1S("%"));
+    setZoom(m_zoomFactor);
 }
 
 
@@ -438,8 +460,25 @@ void WebTab::zoomDefault()
 {
     m_zoomFactor = 10;
     view()->setZoomFactor(QVariant(m_zoomFactor).toReal() / 10);
+    
+    setZoom(m_zoomFactor);
+}
 
-    emit infoToShow(i18n("Default zoom: ") + QString::number(m_zoomFactor * 10) + QL1S("%"));
+
+void WebTab::setZoom(int zoomFactor)
+{
+    // This is needed because of the WebView zoomChanged signal...
+    m_zoomFactor = zoomFactor;
+    
+    // set zoom factor
+    KSharedConfig::Ptr config = KGlobal::config();
+    KConfigGroup group(config, "Zoom");
+    group.writeEntry(url().host(), m_zoomFactor);
+
+    if (m_zoomFactor == 10)
+        emit infoToShow(i18n("Default zoom: %1%", QString::number(m_zoomFactor * 10)));
+    else
+        emit infoToShow(i18n("Zooming: %1%", QString::number(m_zoomFactor * 10)));
 }
 
 
@@ -482,3 +521,31 @@ void WebTab::toggleInspector(bool on)
 
     page()->settings()->setAttribute(QWebSettings::DeveloperExtrasEnabled, on);
 }
+
+
+void WebTab::focusIn()
+{
+#ifdef HAVE_KACTIVITIES
+    if (m_isPrivateBrowsing || !m_activityResourceInstance)
+        return;
+    
+    m_activityResourceInstance->notifyFocusedIn();
+#endif
+}
+
+
+void WebTab::focusOut()
+{
+#ifdef HAVE_KACTIVITIES
+    if (m_isPrivateBrowsing || !m_activityResourceInstance)
+        return;
+    
+    m_activityResourceInstance->notifyFocusedOut();    
+#endif
+}
+
+
+bool WebTab::isWebApp()
+{
+    return m_isWebApp;
+}
