@@ -64,6 +64,7 @@
 #include <QTimer>
 
 #include <QApplication>
+#include <QBitmap>
 #include <QClipboard>
 #include <QContextMenuEvent>
 #include <QLabel>
@@ -72,6 +73,11 @@
 #include <QWebHistory>
 #include <QNetworkRequest>
 
+#ifdef Q_WS_X11
+#include <X11/Xlib.h>
+#include <fixx11h.h>
+#endif
+
 
 // needed for the spellCheck
 static QVariant execJScript(QWebHitTestResult result, const QString& script)
@@ -99,6 +105,7 @@ WebView::WebView(QWidget* parent, bool isPrivateBrowsing)
     , m_isViewSmoothScrolling(false)
     , m_accessKeysPressed(false)
     , m_accessKeysActive(false)
+    , m_isExternalLinkHovered(false)
     , m_parentTab(qobject_cast<WebTab *>(parent))
     , m_isPrivateBrowsing(isPrivateBrowsing)
 {
@@ -134,6 +141,7 @@ void WebView::load(const QNetworkRequest &req, QNetworkAccessManager::Operation
 {
     QNetworkRequest request = req;
     const QUrl &reqUrl = request.url();
+
     if (reqUrl.host() == url().host())
     {
         request.setRawHeader(QByteArray("Referer"), url().toEncoded());
@@ -143,6 +151,20 @@ void WebView::load(const QNetworkRequest &req, QNetworkAccessManager::Operation
 }
 
 
+void WebView::reload()
+{
+    // This is needed to reload empty url, eg when network is down and you
+    // tried to load an URL.
+    if (url().isEmpty())
+    {
+        load(page()->loadingUrl());
+        return;
+    }
+
+    KWebView::reload();
+}
+
+
 void WebView::loadStarted()
 {
     hideAccessKeys();
@@ -154,7 +176,7 @@ WebPage *WebView::page()
     WebPage *p = qobject_cast<WebPage *>(KWebView::page());
     if (!p)
     {
-        p = new WebPage(this, m_isPrivateBrowsing);
+        p = new WebPage(m_isPrivateBrowsing, this);
         setPage(p);
     }
     return p;
@@ -278,7 +300,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
     }
 
     WebWindow *webwin = m_parentTab->webWindow();
-
+    
     KMenu menu(this);
 
     KAction *sendByMailAction = new KAction(&menu);
@@ -315,7 +337,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
     {
         // send by mail: page url
         sendByMailAction->setData(page()->currentFrame()->url());
-        sendByMailAction->setText(i18n("Share page url"));
+        sendByMailAction->setText(i18n("Share page URL"));
 
         // navigation
         QWebHistory *history = page()->history();
@@ -372,6 +394,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         sendByMailAction->setData(m_contextMenuHitResult.linkUrl());
         sendByMailAction->setText(i18n("Share link"));
 
+
         a = new KAction(KIcon("tab-new"), i18n("Open in New &Tab"), &menu);
         a->setData(m_contextMenuHitResult.linkUrl());
         connect(a, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewTab()));
@@ -382,7 +405,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         connect(a, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewWindow()));
         menu.addAction(a);
 
-        if (webwin)
+        if (!m_parentTab->isWebApp())
         {
             a = new KAction(KIcon("view-media-artist"), i18n("Open in Private &Window"), &menu);
             a->setData(m_contextMenuHitResult.linkUrl());
@@ -390,6 +413,14 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
             menu.addAction(a);
         }
 
+        if (m_isExternalLinkHovered)
+        {
+            a = new KAction(KIcon("view-close"), i18n("Open &Here"), this);
+            a->setData(m_contextMenuHitResult.linkUrl());
+            connect(a, SIGNAL(triggered(bool)), this, SLOT(openLinkHere()));
+            menu.addAction(a);
+        }
+
         menu.addSeparator();
 
         // Don't show dots if we are NOT going to ask for download path
@@ -482,7 +513,8 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
                 openInNewTabAction->setData( QUrl::fromUserInput(selectedText()) );
                 connect(openInNewTabAction, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewTab()));
                 menu.addAction(openInNewTabAction);
-                //open selected text url in a new window
+                
+                // open selected text url in a new window
                 QAction * const openInNewWindowAction = new KAction(KIcon("window-new"),
                         i18n("Open '%1' in New Window", truncatedUrl), &menu);
                 openInNewWindowAction->setData( QUrl::fromUserInput(selectedText()) );
@@ -516,10 +548,13 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
             searchMenu->addAction(a);
         }
 
-        a = new KAction(KIcon("edit-find"), i18n("On Current Page"), &menu);
-        connect(a, SIGNAL(triggered()), webwin, SLOT(findSelectedText()));
-        searchMenu->addAction(a);
-
+        if (webwin)
+        {
+            a = new KAction(KIcon("edit-find"), i18n("On Current Page"), &menu);
+            connect(a, SIGNAL(triggered()), webwin, SLOT(findSelectedText()));
+            searchMenu->addAction(a);
+        }
+        
         if (!searchMenu->menu()->isEmpty())
         {
             menu.addAction(searchMenu);
@@ -537,7 +572,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
     }
     else
     {
-        if (webwin)
+        if (!m_parentTab->isWebApp() && webwin)
         {
             a = webwin->actionByName(KStandardAction::name(KStandardAction::AddBookmark));
             menu.addAction(a);
@@ -646,6 +681,8 @@ void WebView::mouseMoveEvent(QMouseEvent *event)
 {
     QPoint mousePos = event->pos();
 
+    guessHoveredLink(mousePos);
+
     if (m_isViewAutoScrolling)
     {
         QPoint r = mousePos - m_clickPos;
@@ -759,6 +796,15 @@ void WebView::slotCopyImageLocation()
 }
 
 
+void WebView::openLinkHere()
+{
+    KAction *a = qobject_cast<KAction*>(sender());
+    KUrl url(a->data().toUrl());
+
+    emit loadUrl(url, Rekonq::CurrentTab);
+}
+
+
 void WebView::openLinkInNewWindow()
 {
     KAction *a = qobject_cast<KAction*>(sender());
@@ -773,12 +819,10 @@ void WebView::openLinkInNewTab()
     KAction *a = qobject_cast<KAction*>(sender());
     KUrl url(a->data().toUrl());
 
-    WebWindow *webwin = m_parentTab->webWindow();
-
-    if (webwin)
-        emit loadUrl(url, Rekonq::NewTab);
-    else
+    if (m_parentTab->isWebApp())
         emit loadUrl(url, Rekonq::NewFocusedTab);
+    else
+        emit loadUrl(url, Rekonq::NewTab);
 }
 
 
@@ -1511,3 +1555,33 @@ void WebView::saveImage()
             QString(),
             !settings()->testAttribute(QWebSettings::PrivateBrowsingEnabled));
 }
+
+
+void WebView::guessHoveredLink(QPoint p)
+{
+    QWebHitTestResult hitTest = page()->mainFrame()->hitTestContent(p);
+    const QUrl url = hitTest.linkUrl();
+    bool emptyUrl = url.isEmpty();
+
+    if (!m_isExternalLinkHovered && emptyUrl)
+        return;
+
+    // if url is empty, reset
+    if (emptyUrl)
+    {
+        kDebug() << "EMPTY LINK";
+        m_isExternalLinkHovered = false;
+        return;
+    }
+
+    QWebFrame *frame = hitTest.linkTargetFrame();
+    if (!frame && m_isExternalLinkHovered)
+        return;
+
+    if (!frame)
+    {
+        kDebug() << "EXTERNAL LINK";
+        m_isExternalLinkHovered = true;
+        return;
+    }
+}
