@@ -31,13 +31,16 @@
 #include "rekonq.h"
 
 #include "application.h"
-#include "tabwindow.h"
+#include "rekonqwindow.h"
+
+#include "tabwidget.h"
 
 #include "adblockmanager.h"
 #include "bookmarkmanager.h"
 #include "iconmanager.h"
 #include "syncmanager.h"
 #include "useragentmanager.h"
+#include "sessionmanager.h"
 
 #include "webpage.h"
 #include "webtab.h"
@@ -124,6 +127,7 @@ WebWindow::WebWindow(QWidget *parent, bool isPrivateBrowsing, WebPage *pg)
 
     // things changed signals
     connect(_tab, SIGNAL(titleChanged(QString)), this, SIGNAL(titleChanged(QString)));
+    connect(_tab, SIGNAL(urlChanged(QUrl)), this, SIGNAL(urlChanged(QUrl)));
     connect(_tab, SIGNAL(iconChanged()), this, SIGNAL(iconChanged()));
 
     // check view signals
@@ -147,7 +151,7 @@ WebWindow::WebWindow(QWidget *parent, bool isPrivateBrowsing, WebPage *pg)
 
     updateHistoryActions();
 
-    if (parent && parent->isFullScreen())
+    if (window()->isFullScreen())
         setWidgetsHidden(true);
 }
 
@@ -215,6 +219,20 @@ void WebWindow::setupActions()
     KStandardAction::keyBindings(this, SLOT(keyBindings()), actionCollection());
     KStandardAction::quit(rApp, SLOT(queryQuit()), actionCollection());
 
+    // WebPage Actions
+    QAction *qa;
+    
+    qa = _tab->page()->action(QWebPage::Undo);
+    actionCollection()->addAction(QL1S("edit_undo"), qa);
+    qa = _tab->page()->action(QWebPage::Redo);
+    actionCollection()->addAction(QL1S("edit_redo"), qa);
+    qa = _tab->page()->action(QWebPage::Cut);
+    actionCollection()->addAction(QL1S("edit_cut"), qa);
+    qa = _tab->page()->action(QWebPage::Copy);
+    actionCollection()->addAction(QL1S("edit_copy"), qa);
+    qa = _tab->page()->action(QWebPage::Paste);
+    actionCollection()->addAction(QL1S("edit_paste"), qa);
+    
     // Configure Main Toolbar
     a = new KAction(KIcon("configure-toolbars"), i18n("Configure Main ToolBar"), this);
     actionCollection()->addAction(QL1S("configure_main_toolbar"), a);
@@ -276,6 +294,11 @@ void WebWindow::setupActions()
     actionCollection()->addAction(QL1S("open_location"), a);
     connect(a, SIGNAL(triggered(bool)) , this, SLOT(openLocation()));
 
+     // User sessions management
+    a = new KAction(KIcon("view-choose"), i18n("&Manage Sessions"), this);
+    actionCollection()->addAction(QL1S("session_manage"), a);
+    connect(a, SIGNAL(triggered(bool)), SessionManager::self(), SLOT(manageSessions()));
+
     // ===== Tools Actions =================================
     a = new KAction(i18n("View Page S&ource"), this);
     a->setIcon(KIcon("application-xhtml+xml"));
@@ -816,7 +839,7 @@ void WebWindow::viewPageSource()
 //         srcTab->page()->setIsOnRekonqPage(true);
 //         srcTab->setPart(pa, tmpUrl);
 //         srcTab->urlBar()->setQUrl(url.pathOrUrl());
-//         m_view->setTabText(m_view->currentIndex(), i18n("Source of: ") + url.prettyUrl());
+//         m_view->setTabText(m_view->currentIndex(), i18n("Source of: %1", url.prettyUrl()));
 //         updateHistoryActions();
 //     }
 
@@ -926,13 +949,13 @@ void WebWindow::showCrashMessageBar()
 
 void WebWindow::openNewWindow()
 {
-    rApp->loadUrl(QUrl("about:home"), Rekonq::NewWindow);
+    rApp->loadUrl(QUrl("rekonq:home"), Rekonq::NewWindow);
 }
 
 
 void WebWindow::checkFocus()
 {
-    if (url().scheme() == QL1S("about"))
+    if (url().scheme() == QL1S("rekonq"))
         _bar->setFocus();
     else
         _tab->view()->setFocus();
@@ -941,27 +964,25 @@ void WebWindow::checkFocus()
 
 void WebWindow::openDownloadsPage()
 {
-    rApp->loadUrl(QUrl("about:downloads"), Rekonq::NewFocusedTab);
+    rApp->loadUrl(QUrl("rekonq:downloads"), Rekonq::NewFocusedTab);
 }
 
 
 void WebWindow::openHistoryPage()
 {
-    rApp->loadUrl(QUrl("about:history"), Rekonq::NewFocusedTab);
+    rApp->loadUrl(QUrl("rekonq:history"), Rekonq::NewFocusedTab);
 }
 
 
 void WebWindow::openBookmarksPage()
 {
-    rApp->loadUrl(QUrl("about:bookmarks"), Rekonq::NewFocusedTab);
+    rApp->loadUrl(QUrl("rekonq:bookmarks"), Rekonq::NewFocusedTab);
 }
 
 
 void WebWindow::openHomePage(Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers)
 {
-    KUrl homeUrl = ReKonfig::useNewTabPage()
-                   ? KUrl(QL1S("about:home"))
-                   : KUrl(ReKonfig::homePage());
+    KUrl homeUrl = KUrl(ReKonfig::homePage());
 
     if (buttons == Qt::MidButton || modifiers == Qt::ControlModifier)
         rApp->loadUrl(homeUrl, Rekonq::NewTab);
@@ -993,7 +1014,7 @@ void WebWindow::keyBindings()
     QPointer<KShortcutsDialog> dialog = new KShortcutsDialog(KShortcutsEditor::AllActions, KShortcutsEditor::LetterShortcutsAllowed, this);
 
     dialog->addCollection(actionCollection(), i18n("web window"));
-    TabWindow *tw = rApp->tabWindow();
+    TabWidget *tw = rApp->rekonqWindow()->tabWidget();
     if (tw)
     {
         dialog->addCollection(tw->actionCollection(), i18n("tab window"));
