@@ -37,11 +37,11 @@
 // Qt Includes
 #include <QWidget>
 #include <QAction>
+#include <QUrl>
 
 // Forward Declarations
 class WebPage;
 class WebTab;
-class WebView;
 
 class BookmarkToolBar;
 class FindBar;
@@ -54,7 +54,6 @@ class KToolBar;
 class QLabel;
 class QPixmap;
 class QTimer;
-class QUrl;
 
 
 class WebWindow : public QWidget
@@ -147,6 +146,7 @@ private Q_SLOTS:
 
 Q_SIGNALS:
     void titleChanged(QString);
+    void urlChanged(QUrl);
     void iconChanged();
     
     void loadStarted();
